<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../xsl_util.xsl"/>
<xsl:output method="text" encoding="utf-8" indent="yes" />
<xsl:template match="/">
<xsl:variable name="component_name">
  <xsl:call-template name="replace">
    <xsl:with-param name="text" select="nerv_icde/component/name" />
    <xsl:with-param name="replace" select="' '" />
    <xsl:with-param name="by" select="'_'" />
  </xsl:call-template>
</xsl:variable>
#include "Exam_<xsl:value-of select="$component_name"/>.hh"

#include &lt;Nerv36/nerv_env.hh&gt;

#include &lt;examout.hh&gt;

namespace Exam {

Exam_<xsl:value-of select="$component_name"/>::Exam_<xsl:value-of select="$component_name"/>()
    : state_machine_(0), state_(0), nonsleep(false)
    , interval_sec(1), interval_ns(0)
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    , err_<xsl:value-of select="object_name"/>_conn_(not_conn_<xsl:value-of select="object_name"/>)
  </xsl:for-each>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    , stub_<xsl:value-of select="object_name"/>_(0)
  </xsl:for-each>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    , proxy_<xsl:value-of select="object_name"/>_(0)
  </xsl:for-each>
{
    state_ = Nerv36::Data::Component::Component_state::malloc();
    state_-&gt;state = Nerv36::Data::Component::TERMINATED_;
    state_-&gt;cmd = Nerv36::Data::Component::TERMINATED_;

    state_-&gt;emergency = 0;
    state_-&gt;release = 0;
    state_-&gt;emergency_state = 0;
    state_-&gt;release_state = 1;

    state_-&gt;reset = 0;
    state_-&gt;reset_state = 0;

    frequency(1);
}

Exam_<xsl:value-of select="$component_name"/>::~Exam_<xsl:value-of select="$component_name"/>()
{
    state_-&gt;free();
}

void Exam_<xsl:value-of select="$component_name"/>::start_stub()
{
<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_path">
      <xsl:call-template name="get_method_domain_path">
        <xsl:with-param name="mathod_domain_id" select="../../method_domain_id" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="method_domain_namespace">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$method_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'::'" />
      </xsl:call-template>
    </xsl:variable>
    {
        const char* obj_name_ = Nerv36::env("object", "<xsl:value-of select="object_name"/>");

        // Nerv36::Port port = Nerv36::Method::<xsl:value-of select="substring($method_domain_namespace,3)"/>::STANDARD_PORT_;
        Nerv36::Port port = 0;
        Nerv36::Error err = stub_<xsl:value-of select="object_name"/>_-&gt;start(obj_name_, &amp;port);
        printf("stub_<xsl:value-of select="object_name"/>_-&gt;start(port=%d) = %s\n",(int)port, Nerv36::get_err_str(err) );<xsl:text/>
    }
  </xsl:for-each>
}

void Exam_<xsl:value-of select="$component_name"/>::start_proxy()
{
<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_path">
      <xsl:call-template name="get_method_domain_path">
        <xsl:with-param name="mathod_domain_id" select="../../method_domain_id" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="method_domain_namespace">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$method_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'::'" />
      </xsl:call-template>
    </xsl:variable>
    {
        const char* obj_name_ = Nerv36::env("object", "<xsl:value-of select="object_name"/>");
        Nerv36::Error err = proxy_<xsl:value-of select="object_name"/>_-&gt;start(obj_name_);
        fprintf(examout_, "proxy_<xsl:value-of select="object_name"/>_-&gt;start(%s) = %s\n",
            obj_name_, Nerv36::get_err_str(err));
    }<xsl:text/>
  </xsl:for-each>
}<xsl:text/>



void Exam_<xsl:value-of select="$component_name"/>::stop_stub()
{
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_path">
      <xsl:call-template name="get_method_domain_path">
        <xsl:with-param name="mathod_domain_id" select="../../method_domain_id" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="method_domain_namespace">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$method_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'::'" />
      </xsl:call-template>
    </xsl:variable>
    {
        Nerv36::Error err = stub_<xsl:value-of select="object_name"/>_-&gt;stop();
        fprintf(examout_, "stub_<xsl:value-of select="object_name"/>_-&gt;stop() = %s\n", Nerv36::get_err_str(err) );
    }<xsl:text/>
  </xsl:for-each>
}

void Exam_<xsl:value-of select="$component_name"/>::stop_proxy()
{
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_path">
      <xsl:call-template name="get_method_domain_path">
        <xsl:with-param name="mathod_domain_id" select="../../method_domain_id" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="method_domain_namespace">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$method_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'::'" />
      </xsl:call-template>
    </xsl:variable>
    {
        Nerv36::Error err = proxy_<xsl:value-of select="object_name"/>_-&gt;stop();
        fprintf(examout_, "proxy_<xsl:value-of select="object_name"/>_-&gt;stop() = %s\n", Nerv36::get_err_str(err));
    }<xsl:text/>
  </xsl:for-each>
}<xsl:text/>

  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:if test="type='INFORMATION' and out_data!='(null)'">
              <xsl:variable name="out_data_domain_namespace">
                <xsl:call-template name="get_data_domain_namespace">
                  <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
                </xsl:call-template>
              </xsl:variable>
void Exam_<xsl:value-of select="$component_name"/>::read_<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>()
{
    Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* inf_data_;
    inf_data_ = Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>::malloc();

    Nerv36::Error err = proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.read(inf_data_);

    if( err == 0 ) inf_data_-&gt;print("<xsl:value-of select="out_data"/>");
    inf_data_-&gt;free();
}<xsl:text/>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>

void Exam_<xsl:value-of select="$component_name"/>::read_from_proxy()
{<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:if test="type='INFORMATION' and out_data!='(null)'">
<xsl:text>
    </xsl:text>read_<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>();<xsl:text/>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
}<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id"/>
              </xsl:call-template>
            </xsl:variable>
            <!-- <xsl:variable name="in_data_domain_namespace_dot" select="substring($in_data_domain_namespace,2)"/>
            <xsl:variable name="out_data_domain_namespace_dot" select="substring($out_data_domain_namespace,2)"/> -->
            <xsl:choose>
              <xsl:when test="type='CALL'">
void Exam_<xsl:value-of select="$component_name"/>::<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_call()
{
    Nerv36::Msg_num msg_num = 0;
    fprintf(examout_, "proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_call()\n");<xsl:text/>
                <xsl:if test="in_data='(null)' and out_data='(null)'">
    Nerv36::Error err = proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_call(&amp;msg_num, 99.0);<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data!='(null)' and out_data='(null)'">
    Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* cal_data_;
    cal_data_ = Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>::malloc();
    cal_data_-&gt;set(0);
    Nerv36::Error err = proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_call(&amp;msg_num, cal_data_, 99.0);
    cal_data_-&gt;free();<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data='(null)' and out_data!='(null)'">
    Nerv36::Error err = proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_call(&amp;msg_num, 99.0);<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data!='(null)' and out_data!='(null)'">
    Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* cal_data_;
    cal_data_ = Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>::malloc();
    cal_data_-&gt;set(0);
    Nerv36::Error err = proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_call( &amp;msg_num, cal_data_, 99.0);
    cal_data_-&gt;free();<xsl:text/>
                </xsl:if>
    fprintf(examout_, "proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_call(msg_num=%d) = %s\n", msg_num, Nerv36::get_err_str(err));
}<xsl:text/>
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
void Exam_<xsl:value-of select="$component_name"/>::<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_signal()
{
    Nerv36::Msg_num msg_num = 0;
    // Double tm = 0;<xsl:text/>
    fprintf(examout_, "proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_signal()\n");<xsl:text/>
                <xsl:if test="in_data='(null)'">
    Nerv36::Error err = proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_signal(&amp;msg_num, 99.0);<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data!='(null)'">
    Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* sig_data_;
    sig_data_ = Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>::malloc();
    sig_data_-&gt;set(0);
    Nerv36::Error err = proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_signal(&amp;msg_num, sig_data_, 99.0);
    sig_data_-&gt;free();<xsl:text/>
                </xsl:if>
    fprintf(examout_, "proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_signal(msg_num=%d) = %s\n", msg_num, Nerv36::get_err_str(err));
}<xsl:text/>
              </xsl:when>
              <xsl:when test="type='COMMAND'">
void Exam_<xsl:value-of select="$component_name"/>::<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_command()
{
    fprintf(examout_, "proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.write()\n");<xsl:text/>
                <xsl:if test="in_data='(null)'">
    Nerv36::Error err = proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.write();<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data!='(null)'">
<xsl:text>
    </xsl:text>Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* cmd_data_;
    cmd_data_ = Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>::malloc();
    cmd_data_-&gt;set(0);
    Nerv36::Error err = proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.write(cmd_data_);
    cmd_data_-&gt;free();<xsl:text/>
                </xsl:if>
    fprintf(examout_, "proxy_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.write() = %s\n", Nerv36::get_err_str(err));
}<xsl:text/>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>

void Exam_<xsl:value-of select="$component_name"/>::send_from_proxy()
{<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:choose>
              <xsl:when test="type='CALL'">
<xsl:text>
    </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_call();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
<xsl:text>
    </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_signal();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='COMMAND'">
<xsl:text>
    </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_command();<xsl:text/>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
}<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id"/>
              </xsl:call-template>
            </xsl:variable>
            <!-- <xsl:variable name="in_data_domain_namespace_dot" select="substring($in_data_domain_namespace,2)"/>
            <xsl:variable name="out_data_domain_namespace_dot" select="substring($out_data_domain_namespace,2)"/> -->
            <xsl:choose>
              <xsl:when test="type='CALLBACK'">
void Exam_<xsl:value-of select="$component_name"/>::<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_callback()
{
    Nerv36::Msg_num msg_num = 0;
    Nerv36::Address proxy_addr = stub_<xsl:value-of select="$object_name"/>_-&gt;proxy_addr;
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_callback()\n");<xsl:text/>
                <xsl:if test="out_data='(null)'">
    Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_callback(&amp;msg_num, proxy_addr,99.0);<xsl:text/>
                </xsl:if>
                <xsl:if test="out_data!='(null)'">
    Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* clb_data_;
    clb_data_ = Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>::malloc();
    clb_data_-&gt;set(0);
    Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_callback(&amp;msg_num, proxy_addr, clb_data_,99.0);
    clb_data_-&gt;free();<xsl:text/>
                </xsl:if>
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_callback() = %s\n", Nerv36::get_err_str(err));
}<xsl:text/>
              </xsl:when>
              <xsl:when test="type='SIGNALBACK'">
void Exam_<xsl:value-of select="$component_name"/>::<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_signalback()
{
    Nerv36::Msg_num msg_num = 0;
    Nerv36::Address proxy_addr = stub_<xsl:value-of select="$object_name"/>_-&gt;proxy_addr;
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_signalback()\n");<xsl:text/>
                <xsl:if test="out_data='(null)'">
    Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_signalback(&amp;msg_num, proxy_addr,99.0);<xsl:text/>
                </xsl:if>
                <xsl:if test="out_data!='(null)'">
    Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* sgb_data_;
    sgb_data_ = Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>::malloc();
    sgb_data_-&gt;set(0);
    Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_signalback(&amp;msg_num, proxy_addr, sgb_data_,99.0);
    sgb_data_-&gt;free();<xsl:text/>
                </xsl:if>
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.delegate_signalback() = %s\n", Nerv36::get_err_str(err));
}<xsl:text/>
              </xsl:when>
              <xsl:when test="type='EVENT'">
void Exam_<xsl:value-of select="$component_name"/>::<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_event()
{
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.notify()\n");<xsl:text/>
                <xsl:if test="out_data='(null)'">
    Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.notify();<xsl:text/>
                </xsl:if>
                <xsl:if test="out_data!='(null)'">
    Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* evt_data_;
    evt_data_ = Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>::malloc();
    evt_data_-&gt;set(0);
    Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.notify(evt_data_);
    evt_data_-&gt;free();<xsl:text/>
                </xsl:if>
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.notify() = %s\n", Nerv36::get_err_str(err));
}<xsl:text/>
              </xsl:when>
              <xsl:when test="type='INFORMATION'">
void Exam_<xsl:value-of select="$component_name"/>::<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_information()
{
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.write()\n");<xsl:text/>
                <xsl:if test="out_data='(null)'">
<xsl:text>
    </xsl:text>Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.write();<xsl:text/>
                </xsl:if>
                <xsl:if test="out_data!='(null)'">
<xsl:text>
    </xsl:text>Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* inf_data_;
    inf_data_ = Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>::malloc();
    inf_data_-&gt;set(0);
    Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.write(inf_data_);
    inf_data_-&gt;free();<xsl:text/>
                </xsl:if>
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.write() = %s\n", Nerv36::get_err_str(err));
}<xsl:text/>
              </xsl:when>

              <xsl:when test="type='BROADCAST'">
<xsl:text>

</xsl:text>void Exam_<xsl:value-of select="$component_name"/>::<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_datagram()
{
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.broadcast()\n");<xsl:text/>
                <xsl:if test="out_data='(null)'">
<xsl:text>
    </xsl:text>Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.broadcast();<xsl:text/>
                </xsl:if>
                <xsl:if test="out_data!='(null)'">
<xsl:text>
    </xsl:text>Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* dtg_data_;
    dtg_data_ = Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>::malloc();
    dtg_data_-&gt;set(0);
    Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.broadcast(dtg_data_);
    dtg_data_-&gt;free();<xsl:text/>
                </xsl:if>
    fprintf(examout_, "stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.broadcast() = %s\n", Nerv36::get_err_str(err));
}<xsl:text/>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:if test="type='COMMAND' and in_data!='(null)'">
              <xsl:variable name="data_domain_namespace">
                <xsl:call-template name="get_data_domain_namespace">
                  <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
                </xsl:call-template>
              </xsl:variable>
void Exam_<xsl:value-of select="$component_name"/>::read_<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>()
{
    Nerv36::Data<xsl:value-of select="$data_domain_namespace"/>::<xsl:value-of select="in_data"/>* cmd_dat_;
    cmd_dat_ = Nerv36::Data<xsl:value-of select="$data_domain_namespace"/>::<xsl:value-of select="in_data"/>::malloc();
    Nerv36::Error err = stub_<xsl:value-of select="$object_name"/>_-&gt;<xsl:value-of select="$method_name"/>.read(stub_<xsl:value-of select="$object_name"/>_-&gt;proxy_addr, cmd_dat_);
    if( err == 0 ) cmd_dat_-&gt;print("<xsl:value-of select="in_data"/>");
    cmd_dat_-&gt;free();
}<xsl:text/>

            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>

void Exam_<xsl:value-of select="$component_name"/>::read_from_stub()
{<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:if test="type='COMMAND' and in_data!='(null)'">
<xsl:text>
    </xsl:text>read_<xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>();<xsl:text/>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
}<xsl:text/>

void Exam_<xsl:value-of select="$component_name"/>::send_from_stub()
{<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:choose>
              <xsl:when test="type='CALLBACK'">
<xsl:text>
    </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_callback();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='SIGNALBACK'">
<xsl:text>
    </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_signalback();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='EVENT'">
<xsl:text>
    </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_event();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='INFORMATION'">
<xsl:text>
    </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_information();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='BROADCAST'">
<xsl:text>
    </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_datagram();<xsl:text/>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
}

} // namespace Exam

</xsl:template>
</xsl:stylesheet>
