<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../xsl_util.xsl"/>
<xsl:output method="text" encoding="utf-8" indent="yes" />
<xsl:template match="/">
<xsl:variable name="component_name">
  <xsl:call-template name="replace">
    <xsl:with-param name="text" select="nerv_icde/component/name" />
    <xsl:with-param name="replace" select="' '" />
    <xsl:with-param name="by" select="'_'" />
  </xsl:call-template>
</xsl:variable>
#include "Exam_<xsl:value-of select="$component_name"/>.hh"

#include &lt;examout.hh&gt;
#include &lt;Waregram/_thread.hpp&gt;

namespace Exam {

  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
const char Exam_<xsl:value-of select="$component_name"/>::conn_<xsl:value-of select="object_name"/>[] = "<xsl:value-of select="object_name"/> is connected.";
const char Exam_<xsl:value-of select="$component_name"/>::not_conn_<xsl:value-of select="object_name"/>[] = "<xsl:value-of select="object_name"/> is not connected.";
  </xsl:for-each>

void Exam_<xsl:value-of select="$component_name"/>::on_finalize()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_finalize()\n", cnt);

    stop_stub();  // Stub을 정지시킨다.

  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    delete stub_<xsl:value-of select="object_name"/>_;
    stub_<xsl:value-of select="object_name"/>_ = 0;
  </xsl:for-each>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    delete proxy_<xsl:value-of select="object_name"/>_;
    proxy_<xsl:value-of select="object_name"/>_ = 0;
  </xsl:for-each>

    trans_terminate();
}

void Exam_<xsl:value-of select="$component_name"/>::on_initialize()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_initialize()\n", cnt);

  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    stub_<xsl:value-of select="object_name"/>_ = new Nerv_stub::<xsl:value-of select="../../class_name"/>_exam;<xsl:text/>
  </xsl:for-each>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    proxy_<xsl:value-of select="object_name"/>_ = new Nerv_proxy::<xsl:value-of select="../../class_name"/>_exam;<xsl:text/>
  </xsl:for-each>

    start_stub(); // Stub을 시작한다.
    start_proxy(); // Proxy를 시작한다.

    trans_idle(); // 같은 사이클에 idle도 처리
}

void Exam_<xsl:value-of select="$component_name"/>::on_idle()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_idle()\n", cnt);
}

void Exam_<xsl:value-of select="$component_name"/>::on_shutdown()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_shutdown()\n", cnt);
    trans_idle();
}

void Exam_<xsl:value-of select="$component_name"/>::on_ready()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_ready()\n", cnt);
    trans_standby();
}

void Exam_<xsl:value-of select="$component_name"/>::on_standby()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_standby()\n", cnt);
}

void Exam_<xsl:value-of select="$component_name"/>::on_stop()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_start()\n", cnt);
    trans_standby();
}

void Exam_<xsl:value-of select="$component_name"/>::on_start()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_start()\n", cnt);
    trans_run();
}

void Exam_<xsl:value-of select="$component_name"/>::on_run()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_run()\n", cnt);
    read_from_proxy();
    send_from_proxy();
    read_from_stub();
    send_from_stub();
}

void Exam_<xsl:value-of select="$component_name"/>::on_failure()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_failure()\n", cnt);
}

Error Exam_<xsl:value-of select="$component_name"/>::failure_check()
{
    Error error = 0;
    <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    if( proxy_<xsl:value-of select="object_name"/>_-&gt;is_connected() ) err_<xsl:value-of select="object_name"/>_conn_ = conn_<xsl:value-of select="object_name"/>;
    else {
        err_<xsl:value-of select="object_name"/>_conn_  = not_conn_<xsl:value-of select="object_name"/>;
        error = NETWORK_ERROR;
    }
    </xsl:for-each>
    return error;
}

void Exam_<xsl:value-of select="$component_name"/>::on_failure_check()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_failure_check()\n", cnt);

    // failure 상태에 진입[failure()]할것인지,
    // failure 상태에서 해제[release()] 할것인지 이 함수를 상속받아서 코딩.
    // failure() 함수는 이미 failure일경우 영향이 없는 함수이다.
    // release() 함수는 failure/emergency가 아닐경우 영향이 없는 함수이다.

    switch(state_-&gt;state) {
    case IDLE_:
    case SHUTDOWN_:
    case READY_:
    case STANDBY_:
    case STOP_:
    case START_:
    case RUN_: { // 항상 에러를 체크하는 상태
        failure(failure_check());
    } break;
    default: { // INIT_ , IDLE_ , TERMENATED_ , 항상 에러 없는 상태
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
        err_<xsl:value-of select="object_name"/>_conn_ = "";
  </xsl:for-each>
        failure(0);
    } break;
    }
}

void Exam_<xsl:value-of select="$component_name"/>::on_emergency_rising_edge()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_emergency_rising_edge()\n", cnt);
}

void Exam_<xsl:value-of select="$component_name"/>::on_emergency()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_emergency()\n", cnt);
}

void Exam_<xsl:value-of select="$component_name"/>::on_release_falling_edge()
{
    fprintf(examout_, "\n%zu: Exam_<xsl:value-of select="$component_name"/>::on_release_falling_edge()\n", cnt);
}

void Exam_<xsl:value-of select="$component_name"/>::state_cycle()
{
    on_failure_check();

    if( state_-&gt;reset ) {
        state_-&gt;reset = 0;
        state_-&gt;reset_state = 1;
        notify_state();
    }

    if(state_-&gt;emergency) {
        state_-&gt;emergency = 0;
        state_-&gt;release = 0;
        state_-&gt;emergency_state = 1;
        state_-&gt;release_state = 0;
        on_emergency_rising_edge();
    }
    else if(state_-&gt;release) {
        state_-&gt;emergency = 0;
        state_-&gt;release = 0;
        state_-&gt;emergency_state = 0;
        state_-&gt;release_state = 1;
        on_release_falling_edge();
    }

    if(state_-&gt;emergency_state) {
        on_emergency();
    }


    // cmd가 존재하면 상태를 cmd로 이전하도록 하는 로직을 실행한다.
    // 어느 곳에서는 상태와 cmd가 같아지는 순간 cmd는 클리어한다.
    bool next_time = false;
    if( state_-&gt;cmd &amp;&amp; next_time==false ) {
        State o;
        if( state_-&gt;state == state_-&gt;cmd &amp;&amp; !state_-&gt;reset_state) {
            o=state_-&gt;cmd;
            set_state(state_-&gt;cmd);
        }
        else {
            if( state_-&gt;state == TERMINATED_) {
                o=INIT_; set_state(o); on_initialize();
            }
            else if( state_-&gt;state == FINAL_) {
                o=FINAL_; set_state(FINAL_); on_finalize();
            }
            else if( state_-&gt;state == INIT_) {
                o=INIT_; set_state(INIT_); on_initialize();
            }
            else if( state_-&gt;state == IDLE_) {
                if( state_-&gt;state &gt; state_-&gt;cmd || state_-&gt;reset_state ) {
                    o=FINAL_; set_state(FINAL_); on_finalize();
                }
                else if( state_-&gt;state &lt; state_-&gt;cmd ) {
                    o=READY_; set_state(READY_); on_ready();
                }
                else {
                    o=IDLE_; set_state(IDLE_);
                }
            }
            else if( state_-&gt;state == SHUTDOWN_) {
                o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown();
            }
            else if( state_-&gt;state == READY_) {
                o=READY_; set_state(READY_); on_ready();
            }
            else if( state_-&gt;state == STANDBY_) {
                if( state_-&gt;state &gt; state_-&gt;cmd || state_-&gt;reset_state ) {
                    o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown();
                }
                else if( state_-&gt;state &lt; state_-&gt;cmd ) {
                    o=START_; set_state(START_); on_start();
                }
                else {
                    o=STANDBY_; set_state(STANDBY_);
                }
            }
            else if( state_-&gt;state == STOP_) {
                o=STOP_; set_state(STOP_); on_stop();
            }
            else if( state_-&gt;state == START_) {
                o=START_; set_state(START_); on_start();
            }
            else if( state_-&gt;state == RUN_) {
                o=STOP_; set_state(STOP_); on_stop();
            }
        }

        // 상태 천이를 시도했는데 변하지 않았다면
        if(o == state_-&gt;state) next_time = true;
    }

    // // cmd가 존재하면 상태를 cmd로 이전하도록 하는 로직을 실행한다.
    // // 어느 곳에서는 상태와 cmd가 같아지는 순간 cmd는 클리어한다.
    // bool next_time = false;
    // if( state_-&gt;cmd &amp;&amp; next_time==false ) {
    //     State o;
    //     switch(state_-&gt;state) {
    //     case TERMINATED_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=TERMINATED_; set_state(TERMINATED_); break;
    //         case FINAL_:    o=INIT_; set_state(o); on_initialize(); break;
    //         case INIT_:     o=INIT_; set_state(o); on_initialize(); break;
    //         case IDLE_:     o=INIT_; set_state(o); on_initialize(); break;
    //         case SHUTDOWN_: o=INIT_; set_state(o); on_initialize(); break;
    //         case READY_:    o=INIT_; set_state(o); on_initialize(); break;
    //         case STANDBY_:  o=INIT_; set_state(o); on_initialize(); break;
    //         case STOP_:     o=INIT_; set_state(o); on_initialize(); break;
    //         case START_:    o=INIT_; set_state(o); on_initialize(); break;
    //         case RUN_:      o=INIT_; set_state(o); on_initialize(); break;
    //         default: break;
    //         }
    //     } break;
    //     case FINAL_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case FINAL_:      o=FINAL_; set_state(FINAL_); break;
    //         case INIT_:       o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case IDLE_:       o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case SHUTDOWN_:   o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case READY_:      o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case STANDBY_:    o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case STOP_:       o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case START_:      o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case RUN_:        o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         default: break;
    //         }
    //     } break;
    //     case INIT_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=INIT_; set_state(INIT_); on_initialize(); break;
    //         case FINAL_:      o=INIT_; set_state(INIT_); on_initialize(); break;
    //         case INIT_:       o=INIT_; set_state(INIT_); break;
    //         case IDLE_:       o=INIT_; set_state(INIT_); on_initialize(); break;
    //         case SHUTDOWN_:   o=INIT_; set_state(INIT_); on_initialize(); break;
    //         case READY_:      o=INIT_; set_state(INIT_); on_initialize(); break;
    //         case STANDBY_:    o=INIT_; set_state(INIT_); on_initialize(); break;
    //         case STOP_:       o=INIT_; set_state(INIT_); on_initialize(); break;
    //         case START_:      o=INIT_; set_state(INIT_); on_initialize(); break;
    //         case RUN_:        o=INIT_; set_state(INIT_); on_initialize(); break;
    //         default: break;
    //         }
    //     } break;
    //     case IDLE_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case FINAL_:      o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case INIT_:       o=FINAL_; set_state(FINAL_); on_finalize(); break;
    //         case IDLE_:       o=IDLE_; set_state(IDLE_); break;
    //         case SHUTDOWN_:   o=READY_; set_state(READY_); on_ready(); break;
    //         case READY_:      o=READY_; set_state(READY_); on_ready(); break;
    //         case STANDBY_:    o=READY_; set_state(READY_); on_ready(); break;
    //         case STOP_:       o=READY_; set_state(READY_); on_ready(); break;
    //         case START_:      o=READY_; set_state(READY_); on_ready(); break;
    //         case RUN_:        o=READY_; set_state(READY_); on_ready(); break;
    //         default: break;
    //         }
    //     } break;
    //     case SHUTDOWN_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case FINAL_:      o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case INIT_:       o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case IDLE_:       o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case SHUTDOWN_:   o=SHUTDOWN_; set_state(SHUTDOWN_); break;
    //         case READY_:      o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case STANDBY_:    o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case STOP_:       o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case START_:      o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case RUN_:        o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         default: break;
    //         }
    //     } break;
    //     case READY_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=READY_; set_state(READY_); on_ready(); break;
    //         case FINAL_:      o=READY_; set_state(READY_); on_ready(); break;
    //         case INIT_:       o=READY_; set_state(READY_); on_ready(); break;
    //         case IDLE_:       o=READY_; set_state(READY_); on_ready(); break;
    //         case SHUTDOWN_:   o=READY_; set_state(READY_); on_ready(); break;
    //         case READY_:      o=READY_; set_state(READY_); break;
    //         case STANDBY_:    o=READY_; set_state(READY_); on_ready(); break;
    //         case STOP_:       o=READY_; set_state(READY_); on_ready(); break;
    //         case START_:      o=READY_; set_state(READY_); on_ready(); break;
    //         case RUN_:        o=READY_; set_state(READY_); on_ready(); break;
    //         default: break;
    //         }
    //     } break;
    //     case STANDBY_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case FINAL_:      o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case INIT_:       o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case IDLE_:       o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case SHUTDOWN_:   o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case READY_:      o=SHUTDOWN_; set_state(SHUTDOWN_); on_shutdown(); break;
    //         case STANDBY_:    o=STANDBY_; set_state(STANDBY_); break;
    //         case STOP_:       o=START_; set_state(START_); on_start(); break;
    //         case START_:      o=START_; set_state(START_); on_start(); break;
    //         case RUN_:        o=START_; set_state(START_); on_start(); break;
    //         default: break;
    //         }
    //     } break;
    //     case STOP_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=STOP_; set_state(STOP_); on_stop(); break;
    //         case FINAL_:      o=STOP_; set_state(STOP_); on_stop(); break;
    //         case INIT_:       o=STOP_; set_state(STOP_); on_stop(); break;
    //         case IDLE_:       o=STOP_; set_state(STOP_); on_stop(); break;
    //         case SHUTDOWN_:   o=STOP_; set_state(STOP_); on_stop(); break;
    //         case READY_:      o=STOP_; set_state(STOP_); on_stop(); break;
    //         case STANDBY_:    o=STOP_; set_state(STOP_); on_stop(); break;
    //         case STOP_:       o=STOP_; set_state(STOP_); break;
    //         case START_:      o=STOP_; set_state(STOP_); on_stop(); break;
    //         case RUN_:        o=STOP_; set_state(STOP_); on_stop(); break;
    //         default: break;
    //         }
    //     } break;
    //     case START_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=START_; set_state(START_); on_start(); break;
    //         case FINAL_:      o=START_; set_state(START_); on_start(); break;
    //         case INIT_:       o=START_; set_state(START_); on_start(); break;
    //         case IDLE_:       o=START_; set_state(START_); on_start(); break;
    //         case SHUTDOWN_:   o=START_; set_state(START_); on_start(); break;
    //         case READY_:      o=START_; set_state(START_); on_start(); break;
    //         case STANDBY_:    o=START_; set_state(START_); on_start(); break;
    //         case STOP_:       o=START_; set_state(START_); on_start(); break;
    //         case START_:      o=START_; set_state(START_); break;
    //         case RUN_:        o=START_; set_state(START_); on_start(); break;
    //         default: break;
    //         }
    //     } break;
    //     case RUN_: {
    //         switch(state_-&gt;cmd) {
    //         case TERMINATED_: o=STOP_; set_state(STOP_); on_stop(); break;
    //         case FINAL_:      o=STOP_; set_state(STOP_); on_stop(); break;
    //         case INIT_:       o=STOP_; set_state(STOP_); on_stop(); break;
    //         case IDLE_:       o=STOP_; set_state(STOP_); on_stop(); break;
    //         case SHUTDOWN_:   o=STOP_; set_state(STOP_); on_stop(); break;
    //         case READY_:      o=STOP_; set_state(STOP_); on_stop(); break;
    //         case STANDBY_:    o=STOP_; set_state(STOP_); on_stop(); break;
    //         case STOP_:       o=STOP_; set_state(STOP_); on_stop(); break;
    //         case START_:      o=STOP_; set_state(STOP_); on_stop(); break;
    //         case RUN_:        o=RUN_; set_state(RUN_); break;
    //         default: break;
    //         }
    //     } break;
    //     default:
    //         // TERMINATED_
    //         break;
    //     }
    //     // 상태 천이를 시도했는데 변하지 않았다면
    //     if(o == state_-&gt;state) next_time = true;
    // }

    // next_time이 true라는 것은 같은
    // cmd에 의한 상태천이중 실행한 상태를 또 실행하는 것이기 때문에
    // 정상실행구간을 실행하지 않는다.
    if(next_time==false) {
        switch(state_-&gt;state) {
        case INIT_: on_initialize(); break;
        case IDLE_: on_idle(); break;
        case READY_: on_ready(); break;
        case STANDBY_: on_standby(); break;
        case START_: on_start(); break;
        case RUN_: on_run(); break;
        case STOP_: on_stop(); break;
        case SHUTDOWN_: on_shutdown(); break;
        default:
            // TERMINATED_
            break;
        }
    }
}

void Exam_<xsl:value-of select="$component_name"/>::state_machine()
{
    trans_initialize();

    auto nano_itvl = std::chrono::nanoseconds{interval_ns};
    auto deadline = std::chrono::steady_clock::now();

    for(cnt = 1; state_machine_; cnt++) {
        state_cycle();
        fflush(examout_);
        if(!nonsleep) {
            deadline += nano_itvl;
            std::this_thread::sleep_until(deadline);
            // if(interval_ns) sleep_for(nanoseconds(interval_ns));
            // if(interval_sec) sleep_for(seconds(interval_sec));
        }
    }
    finalize();
    if( state_-&gt;state == RUN_ ) {
        trans_stop();
    }
    if( state_-&gt;state == STANDBY_ ) {
        trans_shutdown();
    }
    if( state_-&gt;state == IDLE_ ) {
        trans_finalize();
    }
}

void Exam_<xsl:value-of select="$component_name"/>::start_state_machine()
{
    if(state_machine_ == 0) {
        printf("state machine thread\n");
        state_machine_ =
            new std::thread(&amp;Exam_<xsl:value-of select="$component_name"/>::state_machine, this);
    }
}

void Exam_<xsl:value-of select="$component_name"/>::stop_state_machine()
{
    wg_delete_join_thread(state_machine_);
}

void Exam_<xsl:value-of select="$component_name"/>::frequency(int _hz)
{
    interval_ns = (int)(1000000000.0 / _hz);
    interval_sec = 0;
}

void Exam_<xsl:value-of select="$component_name"/>::interval(int _sec)
{
    interval_ns = 0;
    interval_sec = _sec;
}

} // namepsace Exam

</xsl:template>
</xsl:stylesheet>
