<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>
<xsl:param name = "export_data_domain_path" />


<xsl:template match="/">
#pragma once
#include &lt;boost/python.hpp&gt;
//#include &lt;boost/python/reference_existing_object.hpp&gt;
namespace py = boost::python;
#include &lt;Nerv36/Data/copy.hpp&gt;

  <xsl:for-each select="nerv_icde/datadomain/row">
    <xsl:if test="path=$export_data_domain_path">
// data header
#include "<xsl:value-of select="name"/>.h"

// 구조체 배열을 사용하기 위한 array_proxy의 스펙을 맞추기 위해 operator == 정의가 필요함
// xslt로 구조체 배열을 갖는 구조체를 선별해서 단 한번만 operator == 를 정의하기에 어려움이 있음
// 우선 모든 구조체의 operator == () 를 구현하도록 함..
// xslt 에서 런타임에 값을 저장/참조할 수 있는 방법 찾으면 다시 수정할 예정 (javascript)
      <xsl:variable name="open_namespace_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="' { namespace '" />
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="close_namespace_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="' */ } /* '" />
        </xsl:call-template>
      </xsl:variable>

namespace Nerv36 { namespace Data <xsl:value-of select="$open_namespace_path" /> {

#define GET_SCALED(TYPE, ATTR)
#define SET_SCALED(TYPE, ATTR)
#define ADD_PROPERTY_SCALED(TYPE, ATTR) add_property(#ATTR, GET_SCALED(TYPE, ATTR), SET_SCALED(TYPE, ATTR))

    // data export function for boost_python
    void export_python_data()
    {
      <xsl:call-template name="python_scope_define">
        <xsl:with-param name="path" select="concat('/Data',path)" />
      </xsl:call-template>

      <xsl:for-each select="data/row">
        <xsl:if test="type='struct'">
          <xsl:variable name="struct_name" select="name" />
        py::class_&lt;<xsl:value-of select="$struct_name"/> /*, boost::noncopyable */ &gt;("<xsl:value-of select="$struct_name"/>")
            .def("malloc", &amp;<xsl:value-of select="$struct_name"/>::malloc, py::return_value_policy &lt;py::reference_existing_object&gt;() )
            .staticmethod("malloc")
            .def("realloc", &amp;<xsl:value-of select="$struct_name"/>::realloc, py::return_value_policy &lt;py::reference_existing_object&gt;() )
            .def("free", &amp;<xsl:value-of select="$struct_name"/>::free )
            .def("__copy__", &amp;generic__copy__&lt; <xsl:value-of select="$struct_name"/> &gt;)
            .def("__deepcopy__", &amp;generic__deepcopy__&lt; <xsl:value-of select="$struct_name"/> &gt;)
            .def("copy_from", &amp;<xsl:value-of select="$struct_name"/>::copy_from  )
            // .def(init&lt; const <xsl:value-of select="$struct_name"/> &amp; &gt;()) // 복사생성자
          <xsl:for-each select="datafield/row">
            <xsl:variable name="real_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="real_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="member_type">
              <xsl:choose>
                <xsl:when test="type='u_scaled64_t'">::Nerv36::real&lt; ::Nerv36::uInt64, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                <xsl:when test="type='u_scaled32_t'">::Nerv36::real&lt; ::Nerv36::uInt32, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                <xsl:when test="type='u_scaled16_t'">::Nerv36::real&lt; ::Nerv36::uInt16, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                <xsl:when test="type='u_scaled8_t'">::Nerv36::real&lt; ::Nerv36::uInt8, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                <xsl:when test="type='scaled64_t'">::Nerv36::real&lt; ::Nerv36::Int64, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                <xsl:when test="type='scaled32_t'">::Nerv36::real&lt; ::Nerv36::Int32, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                <xsl:when test="type='scaled16_t'">::Nerv36::real&lt; ::Nerv36::Int16, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                <xsl:when test="type='scaled8_t'">::Nerv36::real&lt; ::Nerv36::Int8, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
                <xsl:when test="type='alias'">::Nerv36::Data<xsl:value-of select="$real_data_domain_namespace"/>::<xsl:value-of select="real_data"/></xsl:when>
                <xsl:when test="type='enum8_t'">unsigned int</xsl:when>
                <xsl:when test="type='enum16_t'">unsigned int</xsl:when>
                <xsl:when test="type='enum32_t'">unsigned int</xsl:when>
                <xsl:when test="type='enum64_t'">::Nerv36::uInt64</xsl:when>
                <xsl:when test="type='int8_t'">int</xsl:when>
                <xsl:when test="type='int16_t'">int</xsl:when>
                <xsl:when test="type='int32_t'">int</xsl:when>
                <xsl:when test="type='int64_t'">::Nerv36::Int64</xsl:when>
                <xsl:when test="type='u_int8_t'">unsigned int</xsl:when>
                <xsl:when test="type='u_int16_t'">unsigned int</xsl:when>
                <xsl:when test="type='u_int32_t'">unsigned int</xsl:when>
                <xsl:when test="type='u_int64_t'">::Nerv36::uInt64</xsl:when>
                <xsl:when test="type='ieee754_32_t'">::Nerv36::Float32</xsl:when>
                <xsl:when test="type='ieee754_64_t'">::Nerv36::Float64</xsl:when>
                <xsl:when test="type='char_t'">::Nerv36::Char8</xsl:when>
                <xsl:when test="type='wchar_t'">::Nerv36::Char16</xsl:when>
                <xsl:otherwise>::Nerv36::<xsl:value-of select="type"/></xsl:otherwise>
              </xsl:choose>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test="arraytype='NO'">
                <xsl:choose>
                  <xsl:when test="bitsize!='0'">
            .add_property("<xsl:value-of select="member_name"/>"
                        , +[](const <xsl:value-of select="$struct_name"/>&amp; self) -> <xsl:value-of select="$member_type"/> { return self.<xsl:value-of select="member_name"/>;}
                        , +[](<xsl:value-of select="$struct_name"/>&amp; self, const <xsl:value-of select="$member_type"/> in_val){ self.<xsl:value-of select="member_name"/> = in_val;}
                        )
                  </xsl:when>
                  <xsl:when test="type='scaled64_t' or type='scaled32_t' or type='scaled16_t' or type='scaled8_t' or type='u_scaled64_t' or type='u_scaled32_t' or type='u_scaled16_t' or type='u_scaled8_t'">
            .add_property("<xsl:value-of select="member_name"/>"
                        , +[](const <xsl:value-of select="$struct_name"/>&amp; self) -> double { return (double)self.<xsl:value-of select="member_name"/>;}
                        , +[](<xsl:value-of select="$struct_name"/>&amp; self, const double in_val){ self.<xsl:value-of select="member_name"/> = in_val;}
                        )
                  </xsl:when>
                  <xsl:when test="type='alias'">
                    <xsl:call-template name="alias_member">
                      <xsl:with-param name="struct_name" select="$struct_name"/>
                      <xsl:with-param name="member_name" select="member_name" />
                      <xsl:with-param name="real_domain_id" select="real_domain_id"/>
                      <xsl:with-param name="real_data" select="real_data" />
                    </xsl:call-template>
                  </xsl:when>
                  <xsl:otherwise>
            .def_readwrite("<xsl:value-of select="member_name"/>", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>)
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>

              <xsl:when test="arraytype='FIX'">
                <xsl:choose>
                  <xsl:when test="type!='alias'">
            .def("<xsl:value-of select="member_name"/>_get_at", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_get_at )
            .def("<xsl:value-of select="member_name"/>_set_at", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_set_at )
                  </xsl:when>
                  <xsl:otherwise>
            .def("<xsl:value-of select="member_name"/>_at", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_at, py::return_value_policy&lt;py::reference_existing_object&gt;() )
                  </xsl:otherwise>
                </xsl:choose>
            .def("<xsl:value-of select="member_name"/>_memory", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_memory )
            .def("<xsl:value-of select="member_name"/>_maxsize", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_maxsize )
              </xsl:when>

              <xsl:when test="arraytype='VARIABLE'">
            .def("<xsl:value-of select="member_name"/>_alloc", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_alloc)
            .def("<xsl:value-of select="member_name"/>_size", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_size)
                <xsl:choose>
                  <xsl:when test="type!='alias'">
            .def("<xsl:value-of select="member_name"/>_get_at", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_get_at )
            .def("<xsl:value-of select="member_name"/>_set_at", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_set_at )
                  </xsl:when>
                  <xsl:otherwise>
            .def("<xsl:value-of select="member_name"/>_at", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_at, py::return_value_policy&lt;py::reference_existing_object&gt;() )
                  </xsl:otherwise>
                </xsl:choose>
            .def("<xsl:value-of select="member_name"/>_memory", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_memory )
            .def("<xsl:value-of select="member_name"/>_maxsize", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="member_name"/>_maxsize )
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
            .def("get_sizeof", &amp;<xsl:value-of select="$struct_name"/>::get_sizeof)
            ;
        </xsl:if>
        <xsl:if test="type='alias'">
          <xsl:variable name="real_domain_namespace_empty">
            <xsl:call-template name="get_data_domain_namespace_empty">
              <xsl:with-param name="data_domain_id" select="real_domain_id" />
            </xsl:call-template>
          </xsl:variable>

        {
            PyObject* obj_ = PyImport_ImportModule("py_nerv36Data<xsl:value-of select="$real_domain_namespace_empty"/>");
            py::object nested_module(py::handle&lt;&gt;(py::borrowed(obj_)));
            py::scope real_scope = nested_module;
            parent.attr("<xsl:value-of select="name"/>") = real_scope.attr("<xsl:value-of select="real_data"/>");
        }
        </xsl:if>
      </xsl:for-each>
        } /* <xsl:value-of select="$close_namespace_path" /> */
    }
} } /* <xsl:value-of select="$close_namespace_path" /> */
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<xsl:template name="alias_member">
  <xsl:param name="struct_name"/>
  <xsl:param name="member_name"/>
  <xsl:param name="real_domain_id"/>
  <xsl:param name="real_data"/>
  <xsl:for-each select="/nerv_icde/datadomain/row[id=$real_domain_id]">
    <xsl:for-each select="data/row[name=$real_data]">
      <xsl:choose>
        <xsl:when test="type='alias'">
          <xsl:call-template name="alias_member">
            <xsl:with-param name="struct_name" select="$struct_name"/>
            <xsl:with-param name="member_name" select="$member_name" />
            <xsl:with-param name="real_domain_id" select="real_domain_id"/>
            <xsl:with-param name="real_data" select="real_data" />
          </xsl:call-template>
        </xsl:when>
        <xsl:when test="type='scaled64_t' or type='scaled32_t' or type='scaled16_t' or type='scaled8_t' or type='u_scaled64_t' or type='u_scaled32_t' or type='u_scaled16_t' or type='u_scaled8_t'">
            .add_property("<xsl:value-of select="$member_name"/>"
                        , +[](const <xsl:value-of select="$struct_name"/>&amp; self) -> double { return (double)self.<xsl:value-of select="$member_name"/>;}
                        , +[](<xsl:value-of select="$struct_name"/>&amp; self, const double in_val){ self.<xsl:value-of select="$member_name"/> = in_val;}
                        )
        </xsl:when>
        <xsl:otherwise>
            .def_readwrite("<xsl:value-of select="$member_name"/>", &amp;<xsl:value-of select="$struct_name"/>::<xsl:value-of select="$member_name"/>)
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
  </xsl:for-each>
</xsl:template>

<xsl:template name="python_scope_define">
  <xsl:param name="path" />                                                 <!-- path="/AI/DATA" -->      <!-- path="/DATA" -->

  <xsl:variable name="var" select="substring-after($path,'/')"/>             <!-- var="AI/DATA" -->        <!-- var="DATA" -->

  <xsl:if test="not(contains($var,'/'))">                                                                      <!-- var="DATA" -->
        std::string nested_name = py::extract&lt;std::string&gt;(py::scope().attr("__name__") + "<xsl:value-of select="$var"/>");
        <!-- printf("nested_name = %s\n", nested_name.c_str()); -->
        py::object nested_module(py::handle&lt;&gt;(py::borrowed(PyImport_AddModule(nested_name.c_str()))));
        py::scope().attr("<xsl:value-of select="$var"/>") = nested_module;
        py::scope parent = nested_module;
        {
  </xsl:if>
  <xsl:if test="contains($var,'/')">
    <xsl:variable name="name" select="substring-before($var,'/')"/>            <!-- name="AI" -->
        std::string nested_name = py::extract&lt;std::string&gt;(py::scope().attr("__name__") + "<xsl:value-of select="$name"/>");
        <!-- printf("nested_name = %s\n", nested_name.c_str()); -->
        py::object nested_module(py::handle&lt;&gt;(py::borrowed(PyImport_AddModule(nested_name.c_str()))));
        py::scope().attr("<xsl:value-of select="$name"/>") = nested_module;
        py::scope parent = nested_module;
        {
    <xsl:call-template name="python_scope_define">
      <xsl:with-param name="path" select="concat('/',substring-after($var,'/'))" />     <!-- path="/DATA" -->
    </xsl:call-template>
  </xsl:if>
</xsl:template>

</xsl:stylesheet>
