<?xml version="1.0" encoding="UTF-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_method_domain_path" />

<xsl:template match="/">
  <xsl:for-each select="nerv_icde/methoddomain/row[path=$export_method_domain_path]">
  <xsl:variable name="open_namespace_path">
    <xsl:call-template name="replace">
      <xsl:with-param name="text" select="path" />
      <xsl:with-param name="replace" select="'/'" />
      <xsl:with-param name="by" select="' { namespace '" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="close_namespace_path">
    <xsl:call-template name="replace">
      <xsl:with-param name="text" select="path" />
      <xsl:with-param name="replace" select="'/'" />
      <xsl:with-param name="by" select="' */ } /* '" />
    </xsl:call-template>
  </xsl:variable>
/*******************************************************************************
    Document Header
    domain   : <xsl:value-of select="name"/>
    creator  : <xsl:value-of select="author"/>
    create date          : <xsl:value-of select="create-day"/>
    description          : <xsl:value-of select="description"/>
*******************************************************************************/

#pragma once

/* Include Nerv Sdk Document **************************************************/
#include &lt;Nerv36/Nerv.hh&gt;
#include &lt;boost/python.hpp&gt;
#include &lt;stdio.h&gt;

#include &lt;jsoncpp/json.h&gt;
#include &lt;Waregram/_inet.hpp&gt;
#include &lt;Waregram/_safe_string.hpp&gt;

bool send_log_server(const Json::Value&amp; _json);

extern Socket s;

/* Include Data Domains Document **********************************************/
<xsl:for-each select="ref_data_domain/row">
    <xsl:variable name="ref_domain_id" select="data_domain_id"/>
    <xsl:for-each select="/nerv_icde/datadomain/row">
        <xsl:if test="id=$ref_domain_id">
#include &lt;Nerv36/Data<xsl:value-of select="path"/>/<xsl:value-of select="name"/>.h&gt;
        </xsl:if>
    </xsl:for-each>
</xsl:for-each>

/* Domain Define **************************************************************/

namespace Nerv36 { namespace Method <xsl:value-of select="$open_namespace_path" /> {

    using namespace Nerv36;

    /* Standard Domain Port Constance *****************************************/
    const ::Nerv36::Port c_standard_port = <xsl:value-of select="standard_port"/>;


    /* Method Defines *********************************************************/

<xsl:for-each select="method/row">
    <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
    </xsl:variable>
    /***************************************************************************
        Method       : <xsl:value-of select="name"/>
        Input        : <xsl:if test="in_data != '(null)'"><xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/></xsl:if>
        output       : <xsl:if test="out_data != '(null)'"><xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/></xsl:if>
        Method Type  : <xsl:value-of select="type"/>
        Code         : <xsl:value-of select="code"/>
        Writer       : <xsl:value-of select="../../author"/>
        Description  : <xsl:value-of select="../../description"/>
    ***************************************************************************/
  <xsl:if test="type='CALLBACK'" ><xsl:call-template name="template_callback" /></xsl:if>
  <xsl:if test="type='SIGNALBACK'" ><xsl:call-template name="template_signalback" /></xsl:if>
  <xsl:if test="type='INFORMATION'" ><xsl:call-template name="template_information" /></xsl:if>
  <xsl:if test="type='COMMAND'" ><xsl:call-template name="template_command" /></xsl:if>
  <xsl:if test="type='CALL'" ><xsl:call-template name="template_call" /></xsl:if>
  <xsl:if test="type='SIGNAL'" ><xsl:call-template name="template_signal" /></xsl:if>
  <xsl:if test="type='EVENT'" ><xsl:call-template name="template_event" /></xsl:if>
  <xsl:if test="type='BROADCAST'" ><xsl:call-template name="template_datagram" /></xsl:if>
</xsl:for-each>


    /* Interface Defines ******************************************************/


<xsl:for-each select="interface/row">
    /***************************************************************************
        Interface Name : <xsl:value-of select="name"/>
        Writer : <xsl:value-of select="../../author"/>
        Description : <xsl:value-of select="description"/>
    ***************************************************************************/

    /* proxy */
    class proxy_interface_<xsl:value-of select="name"/>
    : virtual public ::Nerv36::Proxy<xsl:for-each select="membermethod/row">
    <xsl:variable name="method_name" select="method_name"/>
    <xsl:for-each select="../../../../method/row[name=$method_name]">
    , virtual public proxy_method_<xsl:value-of select="$method_name"/>
    </xsl:for-each>
    </xsl:for-each>
    {
    };

    /* stub */
    class stub_interface_<xsl:value-of select="name"/>
    : virtual public ::Nerv36::Stub<xsl:for-each select="membermethod/row">
    <xsl:variable name="method_name" select="method_name"/>
    <xsl:for-each select="../../../../method/row[name=$method_name]">
    , virtual public method_<xsl:value-of select="name"/>
    </xsl:for-each>
    </xsl:for-each>
    {
    };
</xsl:for-each>

} } /* <xsl:value-of select="$close_namespace_path" /> */


</xsl:for-each>
</xsl:template>


<xsl:template name ="template_information" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv36::Proxy_information
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_information(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        ::Nerv36::Error <xsl:value-of select="name"/>_delegate_link(Msg_num_ref&amp; _msg_num_, bool _reliability = false, double _limit = 1.0)
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            ret = delegate_link(&amp;_msg_num_.ref, _reliability , _limit );
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "<xsl:value-of select="name"/>_delegate_link";
                json["error_code"]   = ret;
                (void)send_log_server(json);
            } -->
            return ret;
        }

        ::Nerv36::Error <xsl:value-of select="name"/>_link( bool _reliability = false, double _limit = 1.0, double* _elapsed_ = 0)
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            Promise* promise = async_link( _reliability , _limit );
            ret = await_link_return( promise, _elapsed_ );
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "<xsl:value-of select="name"/>_link";
                json["error_code"]   = ret;
                (void)send_log_server(json);
            } -->
            return ret;
        }

        void <xsl:value-of select="name"/>_unlink()
        {
            Py_BEGIN_ALLOW_THREADS
            unlink();
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "<xsl:value-of select="name"/>_unlink";
                (void)send_log_server(json);
            } -->
        }

        void <xsl:value-of select="name"/>_set_sync(bool _use_on_sync)
        {
            Py_BEGIN_ALLOW_THREADS
            set_sync(_use_on_sync);
            Py_END_ALLOW_THREADS
        }

        Letter_case_ <xsl:value-of select="name"/>_see_history(size_t _size)
        {
            Letter_case_ ret;
            Py_BEGIN_ALLOW_THREADS
            ret = see_history(_size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_set_history_size(size_t _history_size)
        {
            Py_BEGIN_ALLOW_THREADS
            set_history_size( _history_size);
            Py_END_ALLOW_THREADS
        }

        <xsl:if test="out_data!='(null)'">
        Error <xsl:value-of select="name"/>_read( Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_)
        {
            Error ret;
            Nerv36::Pksize inf_size = _inf_dat_->sizeofmax();
            Py_BEGIN_ALLOW_THREADS
            ret = read(_inf_dat_, &amp;inf_size );
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>

    private:

        virtual void on_link(Msg_num _msg_num, Error _err, double _elapsed)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_link";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_link(_msg_num, _err, _elapsed);
        }

        virtual void on_unlink( Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_unlink";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_unlink(_err);
        }

        virtual void on_sync(Sync_model                _sync_model,
                             double                    _elapsed,
                             double                    _limit,
                             const void*               _inf_dat_,
                             Pksize                    _size)
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(proxy_addr.id);
                <!-- json["message_type"] = "information"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>_sync"; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_inf_dat_ );
                </xsl:if>
                (void)send_log_server(json);
            }
            on_<xsl:value-of select="name"/>_sync(_sync_model, _elapsed, _limit <xsl:if test="out_data!='(null)'">, (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_inf_dat_ </xsl:if>);
        }

        virtual void on_<xsl:value-of select="name"/>_unlink(Error _err) = 0;

        virtual void on_<xsl:value-of select="name"/>_sync( Sync_model in_sync_model,
                                                double _elapsed,
                                                double _limit<xsl:if test="out_data!='(null)'">,
                                               const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_ </xsl:if>) = 0;

        virtual void on_<xsl:value-of select="name"/>_link(Msg_num _msg_num, Error _err, double _elapsed) = 0; //{}
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv36::Stub_information
    {
    public:

        method_<xsl:value-of select="name"/>(int interval= 0) : Stub_information(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_set_sync_model( Sync_model _sync_model, double _interval)
        {
            Py_BEGIN_ALLOW_THREADS
            set_sync_model(_sync_model,_interval);
            Py_END_ALLOW_THREADS
        }

        void <xsl:value-of select="name"/>_unlink( Address _proxy_addr )
        {
            Py_BEGIN_ALLOW_THREADS
            unlink( _proxy_addr );
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "<xsl:value-of select="name"/>_unlink";
                (void)send_log_server(json);
            } -->
        }

        void <xsl:value-of select="name"/>_unlink_all_proxy()
        {
            Py_BEGIN_ALLOW_THREADS
            unlink_all_proxy();
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "<xsl:value-of select="name"/>_unlink_all_proxy";
                (void)send_log_server(json);
            } -->
        }

        //
        //
        //
        //
        //
        //

        //
        //
        //
        //
        //
        //

        <xsl:if test="out_data!='(null)'">
        Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* <xsl:value-of select="name"/>_get()
        {
            Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* ret_;
            Py_BEGIN_ALLOW_THREADS
            ret_ = (Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*) get(sizeof(Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>) );
            Py_END_ALLOW_THREADS
            return ret_;
        }

        void <xsl:value-of select="name"/>_cancel()
        {
            Py_BEGIN_ALLOW_THREADS
            cancel();
            Py_END_ALLOW_THREADS
        }

        Error <xsl:value-of select="name"/>_post(Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_)
        {
            Error ret;
            ::Nerv36::Pksize inf_size = nerv_sizeof(_inf_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = post( inf_size);
            Py_END_ALLOW_THREADS

            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string("linked-proxy-everyone");
                <!-- json["message_type"] = "information"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_post"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_inf_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }

            return ret;
        }
        </xsl:if>

        <xsl:if test="out_data!='(null)'">
        Error <xsl:value-of select="name"/>_write(const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_)
        {
            Error ret;
            ::Nerv36::Pksize inf_size = nerv_sizeof(_inf_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = write( _inf_dat_, inf_size);
            Py_END_ALLOW_THREADS

            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string("linked-proxy-everyone");
                <!-- json["message_type"] = "information"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_write"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_inf_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }

            return ret;
        }
        </xsl:if>
        <xsl:if test="out_data='(null)'">
        Error <xsl:value-of select="name"/>_write()
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            ret = write( 0, 0);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string("linked-proxy-everyone");
                <!-- json["message_type"] = "information"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_write"; -->
                <!-- json["error_code"]   = ret; -->
                (void)send_log_server(json);
            }

            return ret;
        }
        </xsl:if>

        Letter_case_ <xsl:value-of select="name"/>_see_history(size_t _size)
        {
            Letter_case_ ret;
            Py_BEGIN_ALLOW_THREADS
            ret = see_history( _size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_set_history_size(size_t _history_size)
        {
            Py_BEGIN_ALLOW_THREADS
            set_history_size(_history_size);
            Py_END_ALLOW_THREADS
        }

        <xsl:if test="out_data!='(null)'">
        Error <xsl:value-of select="name"/>_read( Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_)
        {
            Error ret;
            Nerv36::Pksize inf_size = _inf_dat_->sizeofmax();
            Py_BEGIN_ALLOW_THREADS
            ret = read(_inf_dat_, &amp;inf_size );
            Py_END_ALLOW_THREADS
            return ret;
        }
        </xsl:if>

    private:

        virtual Error on_link( Address _proxy_addr,
                                    bool        _reliability,
                                    double      _elapsed,
                                    double      _limit )
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_link";
                (void)send_log_server(json);
            } -->
            return on_<xsl:value-of select="name"/>_link( _proxy_addr, _reliability, _elapsed, _limit );
        }

        virtual void on_unlink( Address _proxy_addr, Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_unlink";
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_unlink(_proxy_addr, _err);
        }

        virtual Error on_<xsl:value-of select="name"/>_link(Address _proxy_addr,
                                                                 bool        _reliability,
                                                                 double      _elapsed,
                                                                 double      _limit ) = 0;

        virtual void on_<xsl:value-of select="name"/>_unlink( Address _proxy_addr, Error _err) = 0;
    };
</xsl:template>

<xsl:template name ="template_command" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv36::Proxy_command
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_command(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        ::Nerv36::Error <xsl:value-of select="name"/>_delegate_link(Msg_num_ref&amp; _msg_num_, double _limit = 1.0)
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            ret = delegate_link(&amp;_msg_num_.ref, _limit );
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "<xsl:value-of select="name"/>_delegate_link";
                json["error_code"]   = ret;
                (void)send_log_server(json);
            } -->
            return ret;
        }

        ::Nerv36::Error <xsl:value-of select="name"/>_link( double _limit = 1.0, double* _elapsed_ = 0)
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            Promise* promise = async_link( _limit );
            ret = await_link_return( promise, _elapsed_ );
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "<xsl:value-of select="name"/>_link";
                json["error_code"]   = ret;
                (void)send_log_server(json);
            } -->
            return ret;
        }

        void <xsl:value-of select="name"/>_unlink()
        {
            Py_BEGIN_ALLOW_THREADS
            unlink();
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "information";
                json["procedure"]    = "<xsl:value-of select="name"/>_unlink";
                (void)send_log_server(json);
            } -->
        }

        void <xsl:value-of select="name"/>_set_sync_model( bool _reliability,
                                                           Nerv36::Sync_model _sync_model,
                                                           double _interval )
        {
            Py_BEGIN_ALLOW_THREADS
            set_sync_model( _reliability, _sync_model, _interval);
            Py_END_ALLOW_THREADS
        }

    <xsl:if test="in_data!='(null)'">
        Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* <xsl:value-of select="name"/>_get()
        {
            Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* ret_;
            Py_BEGIN_ALLOW_THREADS
            ret_ = (Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)get(sizeof(Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>));
            Py_END_ALLOW_THREADS
            return ret_;
        }

        void <xsl:value-of select="name"/>_cancel()
        {
            Py_BEGIN_ALLOW_THREADS
            cancel();
            Py_END_ALLOW_THREADS
        }

        Error <xsl:value-of select="name"/>_post( const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_)
        {
            Error ret;
            ::Nerv36::Pksize cmd_size = nerv_sizeof(_cmd_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = post( cmd_size);
            Py_END_ALLOW_THREADS

            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "command"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_post"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cmd_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }

            return ret;
        }

        Error <xsl:value-of select="name"/>_write(const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_)
        {
            Error ret;
            ::Nerv36::Pksize cmd_size = nerv_sizeof(_cmd_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = write( _cmd_dat_, cmd_size);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "command"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_write"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cmd_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }

            return ret;
        }

        Error <xsl:value-of select="name"/>_read( Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_)
        {
            Error ret;
            Nerv36::Pksize cmd_size = _cmd_dat_->sizeofmax();
            Py_BEGIN_ALLOW_THREADS
            ret = read(_cmd_dat_, &amp;cmd_size );
            Py_END_ALLOW_THREADS
            return ret;
        }

    </xsl:if>
    <xsl:if test="in_data='(null)'">
        Error <xsl:value-of select="name"/>_write()
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            ret = write( 0, 0);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "command"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_write"; -->
                <!-- json["error_code"]   = ret; -->
                (void)send_log_server(json);
            }
            return ret;
        }
    </xsl:if>
        Letter_case_ <xsl:value-of select="name"/>_see_history(size_t _size)
        {
            Letter_case_ ret;
            Py_BEGIN_ALLOW_THREADS
            ret = see_history( _size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_set_history_size( size_t _history_size)
        {
            Py_BEGIN_ALLOW_THREADS
            set_history_size( _history_size);
            Py_END_ALLOW_THREADS
        }

    private:

        virtual void on_link(Msg_num _msg_num, Error _err, double _elapsed)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "command";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_link";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_link(_msg_num, _err, _elapsed);
        }

        virtual void on_unlink(Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "command";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_unlink";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_unlink(_err);
        }

        virtual void on_<xsl:value-of select="name"/>_unlink(Error _err) = 0;

        virtual void on_<xsl:value-of select="name"/>_link(Msg_num _msg_num, Error _err, double _elapsed) = 0; //{}
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv36::Stub_command
    {
    public:

        method_<xsl:value-of select="name"/>(int interval= 0) : Stub_command(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_unlink( Address _proxy_addr )
        {
            Py_BEGIN_ALLOW_THREADS
            unlink( _proxy_addr );
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "command";
                json["procedure"]    = "<xsl:value-of select="name"/>_unlink";
                (void)send_log_server(json);
            } -->
        }

        void <xsl:value-of select="name"/>_unlink_all_proxy()
        {
            Py_BEGIN_ALLOW_THREADS
            unlink_all_proxy();
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "command";
                json["procedure"]    = "<xsl:value-of select="name"/>_unlink_all_proxy";
                (void)send_log_server(json);
            } -->
        }

        //
        //
        //
        //
        //
        //

        //
        //
        //
        //
        //
        //

        ////////////////////////////////////////////////////////////////////////
        void <xsl:value-of select="name"/>_set_sync(bool _use_on_sync)
        {
            Py_BEGIN_ALLOW_THREADS
            set_sync(_use_on_sync);
            Py_END_ALLOW_THREADS
        }

        Letter_case_ <xsl:value-of select="name"/>_see_history( Address _proxy_addr, size_t _size)
        {
            Letter_case_ ret;
            Py_BEGIN_ALLOW_THREADS
            ret = see_history(_proxy_addr, _size);
            Py_END_ALLOW_THREADS
            return ret;
        }

        void <xsl:value-of select="name"/>_set_history_size( size_t _history_size)
        {
            Py_BEGIN_ALLOW_THREADS
            set_history_size( _history_size);
            Py_END_ALLOW_THREADS
        }
/*
        int <xsl:value-of select="name"/>_get_client_count()
        {
            int ret;
            Py_BEGIN_ALLOW_THREADS
            ret = get_client_count();
            Py_END_ALLOW_THREADS
            return ret;
        }
*/

    <xsl:if test="in_data!='(null)'">
        Error <xsl:value-of select="name"/>_read(Address _proxy_addr, Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_)
        {
            Error ret;
            Nerv36::Pksize cmd_size = _cmd_dat_->sizeofmax();
            Py_BEGIN_ALLOW_THREADS
            ret = read(_proxy_addr, _cmd_dat_, &amp;cmd_size );
            Py_END_ALLOW_THREADS
            return ret;
        }
    </xsl:if>

    private:

        virtual void on_sync(Address _proxy_addr,
                             Sync_model  _sync_model,
                             double      _elapsed,
                             double      _limit,
                             const void* _cmd_dat_,
                             Pksize    _size)
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(_proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "command"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>_sync"; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cmd_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }

            <xsl:if test="in_data!='(null)'">
            on_<xsl:value-of select="name"/>_sync(_proxy_addr, _sync_model,_elapsed, _limit,(const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cmd_dat_);
            </xsl:if>
            <xsl:if test="in_data='(null)'">
            on_<xsl:value-of select="name"/>_sync(_proxy_addr, _sync_model,_elapsed, _limit);
            </xsl:if>
        }

        virtual Error on_link( Address _proxy_addr,
                                       double _elapsed,
                                       double _limit )
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "command";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_link";
                (void)send_log_server(json);
            } -->
            return on_<xsl:value-of select="name"/>_link( _proxy_addr, _elapsed, _limit );
        }

        virtual void on_unlink( Address _proxy_addr, Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "command";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_unlink";
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_unlink(_proxy_addr, _err);
        }

        virtual Error on_<xsl:value-of select="name"/>_link( Address _proxy_addr,
                                                                     double _elapsed,
                                                                     double _limit ) = 0;

        virtual void on_<xsl:value-of select="name"/>_unlink(Address _proxy_addr, Error _err) = 0;



        virtual void on_<xsl:value-of select="name"/>_sync( Address _proxy_addr,
                                                   Sync_model _sync_model,
                                                   double _elapsed,
                                                   double _limit<xsl:if test="in_data!='(null)'">,
                                                   const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_ </xsl:if>) = 0; //{}

    };
</xsl:template>

<xsl:template name ="template_call" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv36::Proxy_call
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_call(<xsl:value-of select="code"/>) {}

        ::Nerv36::Error <xsl:value-of select="name"/>_delegate_call(Msg_num_ref&amp; _msg_num_,<xsl:if test="in_data!='(null)'"> Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cal_dat_, </xsl:if>double _limit = 1.0)
        {
            Error ret;
            <xsl:if test="in_data!='(null)'">::Nerv36::Pksize size = nerv_sizeof(_cal_dat_);</xsl:if>
            Py_BEGIN_ALLOW_THREADS
            ret = delegate_call(&amp;_msg_num_.ref, <xsl:choose><xsl:when test="in_data!='(null)'">_cal_dat_,size,</xsl:when><xsl:when test="in_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit);
            Py_END_ALLOW_THREADS

            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "call"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_delegate_call"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cal_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }

            return ret;
        }

        ::Nerv36::Error <xsl:value-of select="name"/>(<xsl:if test="in_data!='(null)'"> const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cal_dat_,</xsl:if><xsl:if test="out_data!='(null)'"> Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _ret_dat_,</xsl:if> double _limit = 1.0, double* _elapsed_ = 0)
        {
            Error ret;
            <xsl:if test="in_data!='(null)'">::Nerv36::Pksize cal_size = nerv_sizeof(_cal_dat_);</xsl:if>
            <xsl:if test="out_data!='(null)'">::Nerv36::Pksize ret_size = sizeof(Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>);</xsl:if>
            Py_BEGIN_ALLOW_THREADS
            Promise* promise = async_call(<xsl:choose><xsl:when test="in_data!='(null)'">_cal_dat_,cal_size,</xsl:when><xsl:when test="in_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit);
            ret = await_return( promise, <xsl:choose><xsl:when test="out_data!='(null)'">_ret_dat_,&amp;ret_size,</xsl:when><xsl:when test="out_data='(null)'">0, 0,</xsl:when></xsl:choose> _elapsed_ );
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "call(request)"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>"; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cal_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>-return";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(proxy_addr.id);
                <!-- json["message_type"] = "call(response)"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>"; -->
                json["error_code"]   = ret;
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_ret_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }

        void <xsl:value-of select="name"/>_cancel(Error _err)
        {
            Py_BEGIN_ALLOW_THREADS
            cancel(_err);
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "call";
                json["procedure"]    = "<xsl:value-of select="name"/>_cancel";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
        }

    private:

        virtual void on_return(Msg_num     _msg_num,
                           Error    _err,
                           const void* _ret_dat_,
                           Pksize      _ret_size,
                           double      _elapsed)
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>-return";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(proxy_addr.id);
                <!-- json["message_type"] = "call(response)"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>_return"; -->
                <!-- json["error_code"]   = _err; -->
                <xsl:if test="out_data!='(null)'">
                if(_err==0) {
                    Json::Value&amp; data = json["data"].append(Json::Value());
                    Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_ret_dat_ );
                }
                </xsl:if>
                (void)send_log_server(json);
            }

            on_<xsl:value-of select="name"/>_return(_msg_num, _err,<xsl:if test="out_data!='(null)'"> (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_ret_dat_,</xsl:if> _elapsed);
        }

        virtual void on_<xsl:value-of select="name"/>_return(Msg_num _msg_num, Error _err,<xsl:if test="out_data!='(null)'"> const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _ret_dat_,</xsl:if> double _elapsed) = 0; //{}
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv36::Stub_call
    {
    public:

        method_<xsl:value-of select="name"/>() : Stub_call(<xsl:value-of select="code"/>, <xsl:if test="out_data!='(null)'">sizeof(Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>)</xsl:if><xsl:if test="out_data='(null)'">0</xsl:if> ) {}

    protected:
        virtual ::Nerv36::Error on_call(Address  _proxy_addr,
                                             const void*  _cal_dat_,
                                             Pksize     _cal_size,
                                             void*        _ret_dat_,
                                             Pksize*    _ret_size_,
                                             double       _elapsed,
                                             double       _limit )
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(_proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "call(request)"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>"; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cal_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            ::Nerv36::Error r = on_<xsl:value-of select="name"/>(_proxy_addr,<xsl:if test="in_data!='(null)'"> (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_cal_dat_,</xsl:if><xsl:if test="out_data!='(null)'"> (Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_ret_dat_,</xsl:if> _elapsed, _limit);
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>-return";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(_proxy_addr.id);
                <!-- json["message_type"] = "call(response)"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>"; -->
                <!-- json["error_code"]   = r; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_cal_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
  <xsl:if test="out_data!='(null)'">
            if( ! r )
            {
                *_ret_size_ = nerv_sizeof((Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_ret_dat_);
            }
  </xsl:if>
            return r;
        }

        virtual void on_call_cancel( Address _proxy_addr, Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "call";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_cancel";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_cancel(_proxy_addr, _err);
        }

        virtual ::Nerv36::Error on_<xsl:value-of select="name"/>( Address _proxy_addr, <xsl:if test="in_data!='(null)'">const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cal_dat_,</xsl:if> <xsl:if test="out_data!='(null)'">Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _ret_dat_,</xsl:if> double _elapsed, double _limit) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel(Address _proxy_addr, Error _err) = 0;
    };
</xsl:template>

<xsl:template name ="template_signal" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv36::Proxy_signal
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_signal(<xsl:value-of select="code"/>) {}

        ::Nerv36::Error <xsl:value-of select="name"/>_delegate_signal(Msg_num_ref&amp; _msg_num_, <xsl:if test="in_data!='(null)'">const ::Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _sig_dat_,</xsl:if> double _limit = 1.0)
        {
            Error ret;
            <xsl:if test="in_data!='(null)'">::Nerv36::Pksize sig_size = nerv_sizeof(_sig_dat_);</xsl:if>
            Py_BEGIN_ALLOW_THREADS
            ret = delegate_signal(&amp;_msg_num_.ref, <xsl:choose><xsl:when test="in_data!='(null)'">_sig_dat_, sig_size,</xsl:when><xsl:when test="in_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "signal"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_delegate_signal"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_sig_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }

        ::Nerv36::Error <xsl:value-of select="name"/>(<xsl:if test="in_data != '(null)'">const ::Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _sig_dat_,</xsl:if> double _limit = 1.0, double* _elapsed_= 0)
        {
            Error ret;
            <xsl:if test="in_data != '(null)'">::Nerv36::Pksize sig_size = nerv_sizeof(_sig_dat_);</xsl:if>
            Py_BEGIN_ALLOW_THREADS
            ret = signal(<xsl:choose><xsl:when test="in_data != '(null)'">_sig_dat_, sig_size,</xsl:when><xsl:when test="in_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit, _elapsed_);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "signal"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_sig_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }

        void <xsl:value-of select="name"/>_cancel( Error _err)
        {
            Py_BEGIN_ALLOW_THREADS
            cancel(_err);
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "signal";
                json["procedure"]    = "<xsl:value-of select="name"/>_cancel";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
        }

    private:

        virtual void on_return(Msg_num _msg_num, Error _err, double _elapsed)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "signal";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_return";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_return(_msg_num, _err, _elapsed);
        }

        virtual void on_<xsl:value-of select="name"/>_return(Msg_num _msg_num, Error _err, double _elapsed) = 0; // {}
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv36::Stub_signal
    {
    public:
        method_<xsl:value-of select="name"/>() : Stub_signal(<xsl:value-of select="code"/>) {}

    protected:
        virtual void on_signal( Address _proxy_addr,
                                const void* _sig_dat_,
                                Pksize    _sig_size,
                                double      _elapsed )
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(_proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "signal"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>"; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_sig_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            on_<xsl:value-of select="name"/>(_proxy_addr, <xsl:if test="in_data != '(null)'">(const ::Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_sig_dat_,</xsl:if> _elapsed);
        }

        virtual void on_signal_cancel( Address _proxy_addr, Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "signal";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_cancel";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_cancel(_proxy_addr,_err);
        }

        virtual void on_<xsl:value-of select="name"/>( Address _proxy_addr, <xsl:if test="in_data != '(null)'">const ::Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _sig_dat_,</xsl:if> double _elapsed) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel( Address _proxy_addr, Error _err) = 0;
    };
</xsl:template>

<xsl:template name ="template_event" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv36::Proxy_event
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_event(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        ::Nerv36::Error <xsl:value-of select="name"/>_delegate_link(Msg_num_ref&amp; _msg_num_, double _limit = 1.0)
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            ret = delegate_link(&amp;_msg_num_.ref, _limit );
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "event";
                json["procedure"]    = "<xsl:value-of select="name"/>_delegate_link";
                json["error_code"]   = ret;
                (void)send_log_server(json);
            } -->
            return ret;
        }

        ::Nerv36::Error <xsl:value-of select="name"/>_link( double _limit = 1.0, double* _elapsed_= 0 )
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            Promise* promise = async_link( _limit );
            ret = await_link_return( promise, _elapsed_ );
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "event";
                json["procedure"]    = "<xsl:value-of select="name"/>_link";
                json["error_code"]   = ret;
                (void)send_log_server(json);
            } -->
            return ret;
        }

        void <xsl:value-of select="name"/>_unlink()
        {
            Py_BEGIN_ALLOW_THREADS
            unlink();
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "event";
                json["procedure"]    = "<xsl:value-of select="name"/>_unlink";
                (void)send_log_server(json);
            } -->
        }

    private:

        virtual void on_link(Msg_num _msg_num, Error _err, double _elapsed)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "event";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_link";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_link(_msg_num, _err, _elapsed);
        }

        virtual void on_event( const void* _evt_dat_, Pksize _evt_size, double _elapsed)
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(proxy_addr.id);
                <!-- json["message_type"] = "event"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>"; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_evt_dat_ );
                </xsl:if>
                (void)send_log_server(json);
            }
            on_<xsl:value-of select="name"/>(<xsl:if test="out_data!='(null)'">(const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_evt_dat_,</xsl:if> _elapsed);
        }

        virtual void on_unlink(Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "event";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_unlink";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_unlink(_err);
        }

        virtual void on_<xsl:value-of select="name"/>( <xsl:if test="out_data!='(null)'">const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _evt_dat_,</xsl:if> double _elapsed) = 0; //{}

        virtual void on_<xsl:value-of select="name"/>_unlink(Error _err) = 0; //{}

        virtual void on_<xsl:value-of select="name"/>_link(Msg_num _msg_num, Error _err, double _elapsed) = 0; //{}
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv36::Stub_event
    {
    public:
        method_<xsl:value-of select="name"/>() : Stub_event(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_unlink(Address _proxy_addr )
        {
            Py_BEGIN_ALLOW_THREADS
            unlink(_proxy_addr);
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "event";
                json["procedure"]    = "<xsl:value-of select="name"/>_unlink";
                (void)send_log_server(json);
            } -->
        }

        void <xsl:value-of select="name"/>_unlink_all_proxy()
        {
            Py_BEGIN_ALLOW_THREADS
            unlink_all_proxy();
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "event";
                json["procedure"]    = "<xsl:value-of select="name"/>_unlink_all_proxy";
                (void)send_log_server(json);
            } -->
        }

        //
        //
        //
        //
        //
        //

        //
        //
        //
        //
        //
        //

  <xsl:if test="out_data!='(null)'">
        Error <xsl:value-of select="name"/>(const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _evt_dat_)
        {
            Error ret;
            ::Nerv36::Pksize evt_size = nerv_sizeof(_evt_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = notify(_evt_dat_,evt_size);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string("linked-proxy-everyone");
                <!-- json["message_type"] = "event"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_evt_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }
  </xsl:if>
  <xsl:if test="out_data='(null)'">
        Error <xsl:value-of select="name"/>()
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            ret = notify(0,0);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string("linked-proxy-everyone");
                <!-- json["message_type"] = "event"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>"; -->
                <!-- json["error_code"]   = ret; -->
                (void)send_log_server(json);
            }
            return ret;
        }
  </xsl:if>

    private:

        virtual ::Nerv36::Error on_link( Address _proxy_addr,
                                              double _elapsed,
                                              double _limit )
        {
            ::Nerv36::Error ret = on_<xsl:value-of select="name"/>_link(_proxy_addr, _elapsed, _limit);
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "event";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_link";
                json["error_code"]   = ret;
                (void)send_log_server(json);
            } -->
            return ret;
        }

        virtual void on_unlink( Address _proxy_addr, Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "event";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_unlink";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_unlink( _proxy_addr,_err);
        }

        virtual Error on_<xsl:value-of select="name"/>_link( Address _proxy_addr,
                                                      double _elapsed,
                                                      double _limit ) = 0;

        virtual void on_<xsl:value-of select="name"/>_unlink(Address _proxy_addr, Error _err) = 0;
    };
</xsl:template>

<xsl:template name ="template_datagram" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv36::Proxy_datagram
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_datagram(<xsl:value-of select="code"/>)
        {
        }

    private:
        virtual void on_datagram( Address _stub_addr, const void* _dgr_dat_, Pksize _dgr_size, double  _elapsed)
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(_stub_addr.id);
                json["to"] = safe_string(proxy_addr.id);
                <!-- json["message_type"] = "datagram"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>"; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_dgr_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            on_<xsl:value-of select="name"/>(_stub_addr,<xsl:if test="out_data!='(null)'">(const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_dgr_dat_,</xsl:if> _elapsed);
        }
        <xsl:if test="out_data!='(null)'">
        virtual void on_<xsl:value-of select="name"/>( Address _stub_addr, const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _dgr_dat_, double _elapsed) = 0;
        </xsl:if>
        <xsl:if test="out_data='(null)'">
        virtual void on_<xsl:value-of select="name"/>( Address _stub_addr, double _elapsed) = 0;
        </xsl:if>
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv36::Stub_datagram
    {
    public:
        method_<xsl:value-of select="name"/>() : Stub_datagram(<xsl:value-of select="code"/>)
        {
        }

        <xsl:if test="out_data!='(null)'">
        Error <xsl:value-of select="name"/>_broadcast(const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _dgr_dat_)
        {
            Error ret;
            ::Nerv36::Pksize dgr_size = nerv_sizeof(_dgr_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = broadcast(_dgr_dat_,dgr_size);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(_stub_addr.id);
                json["to"] = safe_string("proxy-everyone");
                <!-- json["message_type"] = "datagram"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_broadcast"; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_dgr_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }

        Error <xsl:value-of select="name"/>_cast( ::Nerv36::IP _ip, const ::Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _dgr_dat_)
        {
            Error ret;
            ::Nerv36::Pksize dgr_size = nerv_sizeof(_dgr_dat_);
            Py_BEGIN_ALLOW_THREADS
            ret = cast(_ip, _dgr_dat_,dgr_size);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string("proxy");
                <!-- json["message_type"] = "datagram"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_cast"; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_dgr_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }
        </xsl:if>
        <xsl:if test="out_data='(null)'">
        Error <xsl:value-of select="name"/>_broadcast()
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            ret = broadcast(0,0);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(_stub_addr.id);
                json["to"] = safe_string("proxy-everyone");
                <!-- json["message_type"] = "datagram"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_broadcast"; -->
                (void)send_log_server(json);
            }
            return ret;
        }
        Error <xsl:value-of select="name"/>_cast( ::Nerv36::IP _ip )
        {
            Error ret;
            Py_BEGIN_ALLOW_THREADS
            ret = cast(_ip, 0, 0);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string("proxy");
                <!-- json["message_type"] = "datagram"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_cast"; -->
                (void)send_log_server(json);
            }
            return ret;
        }
        </xsl:if>
    };
</xsl:template>



<xsl:template name ="template_callback" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv36::Proxy_callback
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_callback(<xsl:value-of select="code"/>, <xsl:if test="in_data!='(null)'">sizeof(Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>)</xsl:if><xsl:if test="in_data='(null)'">0</xsl:if>) {}

    private:

        virtual ::Nerv36::Error on_callback( const void*  _clb_dat_,
                                             Pksize       _clb_size,
                                             void*        _ret_dat_,
                                             Pksize*      _ret_size_,
                                             double       _elapsed,
                                             double       _limit )
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(proxy_addr.id);
                <!-- json["message_type"] = "callback(request)"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>"; -->
                <!-- // json["error_code"]   = _err; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_clb_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }

            ::Nerv36::Error r = on_<xsl:value-of select="name"/>(<xsl:if test="out_data!='(null)'"> (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_clb_dat_,</xsl:if><xsl:if test="in_data!='(null)'"> (Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_ret_dat_,</xsl:if> _elapsed, _limit);
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>-return";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "callback(response)"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>"; -->
                <!-- json["error_code"]   = r; -->
                <xsl:if test="in_data!='(null)'">
                if(ret_err==0) {
                    Json::Value&amp; data = json["data"].append(Json::Value());
                    Nerv36::write_json(&amp;data , (Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_ret_dat_ );
                }
                </xsl:if>
                (void)send_log_server(json);
            }
  <xsl:if test="in_data!='(null)'">
            if( ! r )
            {
                *_ret_size_ = nerv_sizeof((Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_ret_dat_);
            }
  </xsl:if>
            return r;
        }

        virtual void on_callback_cancel( Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "callback";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_cancel";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_cancel(_err);
        }

        virtual ::Nerv36::Error on_<xsl:value-of select="name"/>( <xsl:if test="out_data!='(null)'">const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _clb_dat_,</xsl:if> <xsl:if test="in_data!='(null)'">Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _ret_dat_,</xsl:if> double _elapsed, double _limit) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel(Error _err) = 0;

    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv36::Stub_callback
    {
    public:

        method_<xsl:value-of select="name"/>() : Stub_callback(<xsl:value-of select="code"/> ) {}

        ::Nerv36::Error <xsl:value-of select="name"/>_delegate_callback(Msg_num_ref&amp; _msg_num_, Address _proxy_addr, <xsl:if test="out_data!='(null)'"> Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _clb_dat_, </xsl:if>double _limit = 1.0)
        {
            Error ret;
            <xsl:if test="out_data!='(null)'">::Nerv36::Pksize size = nerv_sizeof(_clb_dat_);</xsl:if>
            Py_BEGIN_ALLOW_THREADS
            ret = delegate_callback(&amp;_msg_num_.ref, _proxy_addr, <xsl:choose><xsl:when test="out_data!='(null)'">_clb_dat_, size,</xsl:when><xsl:when test="out_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(_proxy_addr.id);
                <!-- json["message_type"] = "callback"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_delegate_callback"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_clb_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }

        ::Nerv36::Error <xsl:value-of select="name"/>(Address _proxy_addr, <xsl:if test="out_data!='(null)'"> const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _clb_dat_,</xsl:if><xsl:if test="in_data!='(null)'"> Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _ret_dat_,</xsl:if> double _limit = 1.0, double* _elapsed_ = 0)
        {
            Error ret;
            <xsl:if test="out_data!='(null)'">::Nerv36::Pksize clb_size = nerv_sizeof(_clb_dat_);</xsl:if>
            <xsl:if test="in_data!='(null)'">::Nerv36::Pksize ret_size = sizeof(Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>);</xsl:if>
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(_proxy_addr.id);
                <!-- json["message_type"] = "callback(request)"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>"; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_clb_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            Py_BEGIN_ALLOW_THREADS
            Promise* promise = async_callback(_proxy_addr, <xsl:choose><xsl:when test="out_data!='(null)'">_clb_dat_,clb_size,</xsl:when><xsl:when test="out_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit);
            ret = await_return( promise, <xsl:choose><xsl:when test="in_data!='(null)'">_ret_dat_,&amp;ret_size,</xsl:when><xsl:when test="in_data='(null)'">0, 0,</xsl:when></xsl:choose> _elapsed_ );
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>-return";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(_proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "callback(response)"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_ret_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }

        void <xsl:value-of select="name"/>_cancel(Address _proxy_addr, Error _err)
        {
            Py_BEGIN_ALLOW_THREADS
            cancel(_proxy_addr, _err);
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "callback";
                json["procedure"]    = "<xsl:value-of select="name"/>_cancel";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
        }

    private:

        virtual void on_return(Msg_num     _msg_num,
                               Address     _proxy_addr,
                               Error       _err,
                               const void* _ret_dat_,
                               Pksize      _ret_size,
                               double      _elapsed)
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(_proxy_addr.id);
                json["to"] = safe_string(stub_addr.id);
                <!-- json["message_type"] = "callback"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>_return"; -->
                <!-- json["error_code"]   = _err; -->
                <xsl:if test="in_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_ret_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            on_<xsl:value-of select="name"/>_return(_msg_num, _err,<xsl:if test="in_data!='(null)'"> (Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_ret_dat_,</xsl:if> _elapsed);
        }

        virtual void on_<xsl:value-of select="name"/>_return(Msg_num _msg_num, _proxy_addr, Error _err,<xsl:if test="in_data!='(null)'"> Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _ret_dat_,</xsl:if> double _elapsed) = 0; //{}
    };
</xsl:template>


<xsl:template name ="template_signalback" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv36::Proxy_signalback
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_signalback(<xsl:value-of select="code"/>) {}


    private:
        virtual void on_signalback( const void* _sgb_dat_,
                                    Pksize      _sgb_size,
                                    double      _elapsed )
        {
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address proxy_addr,stub_addr;
                get_address(&amp;proxy_addr, &amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(proxy_addr.id);
                <!-- json["message_type"] = "signalback"; -->
                <!-- json["procedure"]    = "on_<xsl:value-of select="name"/>"; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const ::Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_sgb_dat_ );
                </xsl:if>
                (void)send_log_server(json);
            }

            on_<xsl:value-of select="name"/>(<xsl:if test="out_data != '(null)'">(const ::Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_sgb_dat_,</xsl:if> _elapsed);
        }

        virtual void on_signalback_cancel(  Error _err)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "signalback";
                json["procedure"]    = "on_<xsl:value-of select="name"/>_cancel";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_cancel(_err);
        }

        virtual void on_<xsl:value-of select="name"/>( <xsl:if test="out_data != '(null)'">const ::Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _sgb_dat_,</xsl:if> double _elapsed) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel( Error _err) = 0;
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv36::Stub_signalback
    {
    public:
        method_<xsl:value-of select="name"/>() : Stub_signalback(<xsl:value-of select="code"/>) {}


        ::Nerv36::Error <xsl:value-of select="name"/>_delegate_signalback(Msg_num_ref&amp; _msg_num_, Address _proxy_addr, <xsl:if test="out_data!='(null)'">const ::Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _sgb_dat_,</xsl:if> double _limit = 1.0)
        {
            Error ret;
            <xsl:if test="out_data!='(null)'">::Nerv36::Pksize sgb_size = nerv_sizeof(_sgb_dat_);</xsl:if>
            Py_BEGIN_ALLOW_THREADS
            ret = delegate_signalback(&amp;_msg_num_.ref, _proxy_addr, <xsl:choose><xsl:when test="out_data!='(null)'">_sgb_dat_, sgb_size,</xsl:when><xsl:when test="out_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(_proxy_addr.id);
                <!-- json["message_type"] = "signalback"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>_delegate_signalback"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_sgb_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }

        ::Nerv36::Error <xsl:value-of select="name"/>(Address _proxy_addr, <xsl:if test="out_data != '(null)'">const ::Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _sgb_dat_,</xsl:if> double _limit = 1.0, double* _elapsed_= 0)
        {
            Error ret;
            <xsl:if test="out_data != '(null)'">::Nerv36::Pksize sgb_size = nerv_sizeof(_sgb_dat_);</xsl:if>
            Py_BEGIN_ALLOW_THREADS
            ret = signalback(_proxy_addr, <xsl:choose><xsl:when test="out_data != '(null)'">_sgb_dat_, sgb_size,</xsl:when><xsl:when test="out_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit, _elapsed_);
            Py_END_ALLOW_THREADS
            if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                Address stub_addr;
                get_address(&amp;stub_addr);
                json["from"] = safe_string(stub_addr.id);
                json["to"] = safe_string(_proxy_addr.id);
                <!-- json["message_type"] = "signalback"; -->
                <!-- json["procedure"]    = "<xsl:value-of select="name"/>"; -->
                <!-- json["error_code"]   = ret; -->
                <xsl:if test="out_data!='(null)'">
                Json::Value&amp; data   = json["data"].append(Json::Value());
                Nerv36::write_json(&amp;data , (const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_sgb_dat_);
                </xsl:if>
                (void)send_log_server(json);
            }
            return ret;
        }

        void <xsl:value-of select="name"/>_cancel(Address _proxy_addr,  Error _err)
        {
            Py_BEGIN_ALLOW_THREADS
            cancel(_proxy_addr, _err);
            Py_END_ALLOW_THREADS
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "signalback";
                json["procedure"]    = "<xsl:value-of select="name"/>_cancel";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
        }

    private:

        virtual void on_return(Msg_num _msg_num, Error _err, double _elapsed)
        {
            <!-- if(s!=INVALID_SOCKET) {
                Json::Value json;
                json["message_name"] = "<xsl:value-of select="name"/>";
                json["message_type"] = "signalback";
                json["procedure"]    = "<xsl:value-of select="name"/>_return";
                json["error_code"]   = _err;
                (void)send_log_server(json);
            } -->
            on_<xsl:value-of select="name"/>_return(_msg_num, _err, _elapsed);
        }

        virtual void on_<xsl:value-of select="name"/>_return(Msg_num _msg_num, Error _err, double _elapsed) = 0; // {}


    };
</xsl:template>




</xsl:stylesheet>
