#pragma once

#include <assert.h>
#include <stdio.h>

#ifdef TRACE_THREAD
#undef TRACE_THREAD
#endif

#ifdef TRACE
#undef TRACE
#endif

#ifdef SDEF
#undef SDEF
#endif

#ifdef SCALL
#undef SCALL
#endif

#ifdef STRACE
#undef STRACE
#endif

#ifdef ASSERT
#undef ASSERT
#endif

#ifdef VERIFY
#undef VERIFY
#endif

#define __NOW_FUNCTION__ __FUNCTION__ // __FUNCTION__ __PRETTY_FUNCTION__

static inline void wg_nop() {}

#pragma warning(disable : 4819)

#define PRINT(AAA, ...)                                                        \
    {                                                                          \
        fprintf(stdout, AAA, ##__VA_ARGS__);                                   \
        fflush(stdout);                                                        \
    }

#define ASSERT_BUG() assert(false)

// 릴리즈 및 디폴트 디버그 모드(1) 시에는 TRACE 출력 없음
#if !defined(USE_TRACE) || (USE_TRACE == 0)
#define TRACE_THREAD(...) wg_nop()
#define TRACE(...) wg_nop()
#define SDEFVOID
#define SCALVOID
#define TCALVOID
#define SDEF
#define SCAL
#define TCAL
#define STRACE(...) wg_nop()
#define NOTRACE(...) wg_nop()
#define NOSTRACE(...) wg_nop()
#define TRACE_THREAD(...) wg_nop()
#define NOTRACE_THREAD(...) wg_nop()

// 디버그 모드(2) 시에는 간단한 TRACE 출력, STRACE는 TRACE로 작동
#elif USE_TRACE == 1
#include <stdio.h>
#define TRACE_THREAD(AAA, ...)                                                 \
    fprintf(stderr, AAA "\n", ##__VA_ARGS__);                                  \
    fflush(stderr);
#define TRACE(AAA, ...)                                                        \
    fprintf(stderr, AAA "\n", ##__VA_ARGS__);                                  \
    fflush(stderr);
#define SDEFVOID
#define SCALVOID
#define TCALVOID
#define SDEF
#define SCAL
#define TCAL
#define STRACE(AAA, ...)                                                       \
    fprintf(stderr, AAA "\n", ##__VA_ARGS__);                                  \
    fflush(stderr);
#define NOTRACE(...) wg_nop()
#define NOSTRACE(...) wg_nop()
#define NOTRACE_THREAD(...) wg_nop()

// 디버그 모드(3) 시에는 코드위치를 포함하는 TRACE 출력, STRACE는 TRACE로 작동
#elif USE_TRACE == 2
#include <stdio.h>
#define TRACE_THREAD(AAA, ...)                                                 \
    fprintf(stderr, AAA "\n", ##__VA_ARGS__);                                  \
    fflush(stderr);
#define TRACE(AAA, ...)                                                        \
    {                                                                          \
        fprintf(stderr, "%s:%d(%s): " AAA "\n", __FILE__, __LINE__,            \
                __NOW_FUNCTION__, ##__VA_ARGS__);                              \
        fflush(stderr);                                                        \
    }
#define SDEFVOID
#define SCALVOID
#define TCALVOID
#define SDEF
#define SCAL
#define TCAL
#define STRACE(AAA, ...)                                                       \
    {                                                                          \
        fprintf(stderr, "%s:%d(%s): " AAA "\n", __FILE__, __LINE__,            \
                __NOW_FUNCTION__, ##__VA_ARGS__);                              \
        fflush(stderr);                                                        \
    }
#define NOTRACE(...) wg_nop()
#define NOSTRACE(...) wg_nop()
#define NOTRACE_THREAD(...) wg_nop()

// 디버그 모드(4) 시에는 스레드정보 출력, STRACE는 스택 추정기능을 포함함.
#elif USE_TRACE == 3
#include <sstream>
#include <stdio.h>
#include <string>
#include <thread>

extern thread_local char this_thread_name[128];
#define TRACE(AAA, ...)                                                        \
    {                                                                          \
        std::ostringstream ss;                                                 \
        ss << std::this_thread::get_id();                                      \
        std::string sid = ss.str();                                            \
        long long   llid = atoll(sid.c_str());                                 \
        fprintf(stderr, "%s(%llX): %s:%d(%s): " AAA "\n", this_thread_name,    \
                llid, __FILE__, __LINE__, __NOW_FUNCTION__, ##__VA_ARGS__);    \
        fflush(stderr);                                                        \
    }
#define SDEFVOID                                                               \
    const char *_srcfile_, const int _srcline, const char *_srcfunc_
#define SDEF const char *_srcfile_, const int _srcline, const char *_srcfunc_,
#define SCALVOID _srcfile_, _srcline, _srcfunc_
#define SCAL _srcfile_, _srcline, _srcfunc_,
#define TCALVOID __FILE__, __LINE__, __NOW_FUNCTION__
#define TCAL __FILE__, __LINE__, __NOW_FUNCTION__,
#define STRACE(AAA, ...)                                                       \
    {                                                                          \
        std::ostringstream ss;                                                 \
        ss << std::this_thread::get_id();                                      \
        std::string sid = ss.str();                                            \
        long long   llid = atoll(sid.c_str());                                 \
        fprintf(stderr, "%s(%llX): %s:%d(%s) from %s:%d(%s): " AAA "\n",       \
                this_thread_name, llid, __FILE__, __LINE__, __NOW_FUNCTION__,  \
                _srcfile_, _srcline, _srcfunc_, ##__VA_ARGS__);                \
        fflush(stderr);                                                        \
    }
#define TRACE_THREAD(AAA, ...)                                                 \
    snprintf(this_thread_name, 128, "%s", __FUNCTION__);                       \
    TRACE(AAA "", ##__VA_ARGS__);

#define NOTRACE(...) wg_nop()
#define NOSTRACE(...) wg_nop()
#define NOTRACE_THREAD(...) snprintf(this_thread_name, 128, "%s", __FUNCTION__)

#endif

#if !defined(USE_ASSERT) || USE_ASSERT == 0
#define ASSERT(EXP, ...) wg_nop()
#define VERIFY(EXP, ...) (EXP)

#else
#define ASSERT(EXP)                                                            \
    if(!(EXP)) {                                                               \
        for(;;) {                                                              \
            printf("ASSERT Failed: " #EXP ", %s(%d)", __FILE__, __LINE__);      \
            using namespace std::this_thread;                                  \
            using namespace std::chrono;                                       \
            sleep_for(seconds(1));                                             \
        }                                                                      \
    }

#define VERIFY(EXP)                                                            \
    if(!(EXP)) {                                                               \
        for(;;) {                                                              \
            printf("VERIFY Failed: " #EXP ", %s(%d)", __FILE__, __LINE__);      \
            using namespace std::this_thread;                                  \
            using namespace std::chrono;                                       \
            sleep_for(seconds(1));                                             \
        }                                                                      \
    }

#endif