<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" omit-xml-declaration="yes" indent="no"/>
<xsl:param name = "export_data_domain_path" />
<xsl:param name = "export_data_name" />
<xsl:template match="/">
<xsl:variable name="component_name">
  <xsl:call-template name="replace">
    <xsl:with-param name="text" select="nerv_icde/component/name" />
    <xsl:with-param name="replace" select="' '" />
    <xsl:with-param name="by" select="'_'" />
  </xsl:call-template>
</xsl:variable>
#pragma once
  <xsl:for-each select="nerv_icde/datadomain/row[path=$export_data_domain_path]/data/row[name=$export_data_name]">
    <xsl:variable name="underscore_path">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$export_data_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'__'" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="ROS" select="starts-with($export_data_domain_path, '/ROS/')"/>
    <xsl:variable name="ROS_path">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$export_data_domain_path" />
        <xsl:with-param name="replace" select="'/ROS/'" />
        <xsl:with-param name="by" select="''" />
      </xsl:call-template>
    </xsl:variable>

// include Nerv Data header &amp; ROS Data Header
#include &lt;Nerv36/Data<xsl:value-of select="$export_data_domain_path"/>/<xsl:value-of select="$export_data_name"/>.hh&gt;<xsl:text/>

    <xsl:if test="not($ROS)">
#include &lt;<xsl:value-of select="$component_name"/>/<xsl:value-of select="$export_data_name"/>.h&gt;<xsl:text/>
    </xsl:if>
    <xsl:if test="$ROS">
#include &lt;<xsl:value-of select="$ROS_path"/>/<xsl:value-of select="$export_data_name"/>.h&gt;<xsl:text/>
    </xsl:if>


    <xsl:if test="type='alias'">
      <xsl:variable name="real_domain_id" select="real_domain_id"/>
      <xsl:variable name="real_data" select="real_data"/>
      <xsl:for-each select="/nerv_icde/datadomain/row[id=$real_domain_id]">
// include roscopy of alias data
#include &lt;<xsl:value-of select="$component_name"/>/<xsl:value-of select="name"/>_roscopy.hh&gt;<xsl:text/>
      </xsl:for-each>
    </xsl:if>
// include roscopy of alias member
    <xsl:if test="type='struct'">
      <xsl:for-each select="datafield/row[type='alias']">
        <xsl:variable name="real_domain_id" select="real_domain_id"/>
        <xsl:variable name="real_data" select="real_data"/>
        <xsl:for-each select="/nerv_icde/datadomain/row[id=$real_domain_id]">
          <xsl:variable name="filed_alias_underscore_path">
            <xsl:call-template name="replace">
              <xsl:with-param name="text" select="path" />
              <xsl:with-param name="replace" select="'/'" />
              <xsl:with-param name="by" select="'__'" />
            </xsl:call-template>
          </xsl:variable>
#include &lt;<xsl:value-of select="$component_name"/>/<xsl:value-of select="$real_data"/>_roscopy.hh&gt;<xsl:text/>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:if>

    <xsl:variable name="data_scope">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$export_data_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'::'" />
      </xsl:call-template>
    </xsl:variable>


    <xsl:if test="not($ROS)">
inline void copy( <xsl:value-of select="$component_name"/>::<xsl:value-of select="$export_data_name"/>* _dst_, const Nerv36::Data<xsl:value-of select="$data_scope"/>::<xsl:value-of select="$export_data_name"/>* _src_ )<xsl:text/>
    </xsl:if>
    <xsl:if test="$ROS">
inline void copy( <xsl:value-of select="$ROS_path"/>::<xsl:value-of select="$export_data_name"/>* _dst_, const Nerv36::Data<xsl:value-of select="$data_scope"/>::<xsl:value-of select="$export_data_name"/>* _src_ )<xsl:text/>
    </xsl:if>
{
    <xsl:if test="type='alias'">
    copy( &amp;_dst_->msg, _src_ );
    </xsl:if>
    <xsl:if test="type='struct'">
      <xsl:for-each select="datafield/row">
        <xsl:variable name="real_data_domain_namespace">
          <xsl:call-template name="get_data_domain_namespace">
            <xsl:with-param name="data_domain_id" select="real_domain_id" />
          </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="member_type">
          <xsl:choose>
            <xsl:when test="type='u_scaled64_t'">::Nerv36::TReal&lt; ::Nerv36::uInt64, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='u_scaled32_t'">::Nerv36::TReal&lt; ::Nerv36::uInt32, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='u_scaled16_t'">::Nerv36::TReal&lt; ::Nerv36::uInt16, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='u_scaled8_t'">::Nerv36::TReal&lt; ::Nerv36::uInt8, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='scaled64_t'">::Nerv36::TReal&lt; ::Nerv36::Int64, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='scaled32_t'">::Nerv36::TReal&lt; ::Nerv36::Int32, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='scaled16_t'">::Nerv36::TReal&lt; ::Nerv36::Int16, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='scaled8_t'">::Nerv36::TReal&lt; ::Nerv36::Int8, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='alias'">::Nerv36::Data<xsl:value-of select="$real_data_domain_namespace"/>::<xsl:value-of select="real_data"/></xsl:when>
            <xsl:when test="type='int8_t'">::Nerv36::Int8</xsl:when>
            <xsl:when test="type='int16_t'">::Nerv36::Int16</xsl:when>
            <xsl:when test="type='int32_t'">::Nerv36::Int32</xsl:when>
            <xsl:when test="type='int64_t'">::Nerv36::Int64</xsl:when>
            <xsl:when test="type='u_int8_t'">::Nerv36::uInt8</xsl:when>
            <xsl:when test="type='u_int16_t'">::Nerv36::uInt16</xsl:when>
            <xsl:when test="type='u_int32_t'">::Nerv36::uInt32</xsl:when>
            <xsl:when test="type='u_int64_t'">::Nerv36::uInt64</xsl:when>
            <xsl:when test="type='ieee754_32_t'">::Nerv36::Float32</xsl:when>
            <xsl:when test="type='ieee754_64_t'">::Nerv36::Float64</xsl:when>
            <xsl:when test="type='char_t'">::Nerv36::Char8</xsl:when>
            <xsl:when test="type='wchar_t'">::Nerv36::Char16</xsl:when>
            <xsl:otherwise>::Nerv36::<xsl:value-of select="type"/></xsl:otherwise>
           </xsl:choose>
        </xsl:variable>
        <xsl:if test="arraytype='FIX'">
    // copy fix array
          <xsl:choose>
            <xsl:when test="type='alias'">
    for(int i=0;i&lt;<xsl:value-of select="arraysize"/>;i++)
        copy( &amp;_dst_-><xsl:value-of select="member_name"/>[i], &amp;_src_-><xsl:value-of select="member_name"/>[i] );
            </xsl:when>
            <xsl:otherwise>
    memcpy( &amp;_dst_-><xsl:value-of select="member_name"/>, &amp;_src_-><xsl:value-of select="member_name"/> , <xsl:value-of select="arraysize"/>*sizeof(<xsl:value-of select="$member_type"/>) );
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
        <xsl:if test="arraytype='VARIABLE'">
    // copy variable array
          <xsl:choose>
            <xsl:when test="type='alias'">
    _dst_-><xsl:value-of select="member_name"/>.resize(_src_-><xsl:value-of select="member_name"/>_size());
    for(int i=0;i&lt;_src_-><xsl:value-of select="member_name"/>_size();i++)
        copy( &amp;_dst_-><xsl:value-of select="member_name"/>[i], &amp;_src_-><xsl:value-of select="member_name"/>()[i] );
            </xsl:when>
            <xsl:when test="type='char_t'">
    _dst_-><xsl:value-of select="member_name"/> = (char*)_src_-><xsl:value-of select="member_name"/>();
            </xsl:when>
            <xsl:otherwise>
    _dst_-><xsl:value-of select="member_name"/>.resize(_src_-><xsl:value-of select="member_name"/>_size());
    memcpy( _dst_-><xsl:value-of select="member_name"/>.data(), _src_-><xsl:value-of select="member_name"/>() , _src_-><xsl:value-of select="member_name"/>_size()*sizeof(<xsl:value-of select="$member_type"/>) );
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
        <xsl:if test="arraytype='NO'">
          <xsl:choose>
            <xsl:when test="type='alias'">
    copy( &amp;_dst_-><xsl:value-of select="member_name"/>, &amp;_src_-><xsl:value-of select="member_name"/> );
            </xsl:when>
            <xsl:otherwise>
    _dst_-><xsl:value-of select="member_name"/> = _src_-><xsl:value-of select="member_name"/>;
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
      </xsl:for-each>
    </xsl:if>
}

    <xsl:if test="not($ROS)">
inline void copy( Nerv36::Data<xsl:value-of select="$data_scope"/>::<xsl:value-of select="$export_data_name"/>* _dst_, const <xsl:value-of select="$component_name"/>::<xsl:value-of select="$export_data_name"/>* _src_)<xsl:text/>
    </xsl:if>
    <xsl:if test="$ROS">
inline void copy( Nerv36::Data<xsl:value-of select="$data_scope"/>::<xsl:value-of select="$export_data_name"/>* _dst_, const <xsl:value-of select="$ROS_path"/>::<xsl:value-of select="$export_data_name"/>* _src_)<xsl:text/>
    </xsl:if>
{
    <xsl:if test="type='alias'">
    copy( _dst_, &amp;_src_->msg );
    </xsl:if>
    <xsl:if test="type='struct'">
      <xsl:for-each select="datafield/row">
        <xsl:variable name="real_data_domain_namespace">
          <xsl:call-template name="get_data_domain_namespace">
            <xsl:with-param name="data_domain_id" select="real_domain_id" />
          </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="member_type">
          <xsl:choose>
            <xsl:when test="type='u_scaled64_t'">::Nerv36::TReal&lt; ::Nerv36::uInt64, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='u_scaled32_t'">::Nerv36::TReal&lt; ::Nerv36::uInt32, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='u_scaled16_t'">::Nerv36::TReal&lt; ::Nerv36::uInt16, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='u_scaled8_t'">::Nerv36::TReal&lt; ::Nerv36::uInt8, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='scaled64_t'">::Nerv36::TReal&lt; ::Nerv36::Int64, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='scaled32_t'">::Nerv36::TReal&lt; ::Nerv36::Int32, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='scaled16_t'">::Nerv36::TReal&lt; ::Nerv36::Int16, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='scaled8_t'">::Nerv36::TReal&lt; ::Nerv36::Int8, min_<xsl:value-of select="member_name"/>, max_<xsl:value-of select="member_name"/> &gt;</xsl:when>
            <xsl:when test="type='alias'">::Nerv36::Data<xsl:value-of select="$real_data_domain_namespace"/>::<xsl:value-of select="real_data"/></xsl:when>
            <xsl:when test="type='int8_t'">::Nerv36::Int8</xsl:when>
            <xsl:when test="type='int16_t'">::Nerv36::Int16</xsl:when>
            <xsl:when test="type='int32_t'">::Nerv36::Int32</xsl:when>
            <xsl:when test="type='int64_t'">::Nerv36::Int64</xsl:when>
            <xsl:when test="type='u_int8_t'">::Nerv36::uInt8</xsl:when>
            <xsl:when test="type='u_int16_t'">::Nerv36::uInt16</xsl:when>
            <xsl:when test="type='u_int32_t'">::Nerv36::uInt32</xsl:when>
            <xsl:when test="type='u_int64_t'">::Nerv36::uInt64</xsl:when>
            <xsl:when test="type='ieee754_32_t'">::Nerv36::Float32</xsl:when>
            <xsl:when test="type='ieee754_64_t'">::Nerv36::Float64</xsl:when>
            <xsl:when test="type='char_t'">::Nerv36::Char8</xsl:when>
            <xsl:when test="type='wchar_t'">::Nerv36::Char16</xsl:when>
            <xsl:otherwise>::Nerv36::<xsl:value-of select="type"/></xsl:otherwise>
           </xsl:choose>
        </xsl:variable>
        <xsl:if test="arraytype='FIX'">
    // copy fix array
          <xsl:choose>
            <xsl:when test="type='alias'">
    for(int i=0;i&lt;<xsl:value-of select="arraysize"/>;i++)
        copy( &amp;_dst_-><xsl:value-of select="member_name"/>[i], &amp;_src_-><xsl:value-of select="member_name"/>[i] );
            </xsl:when>
            <xsl:otherwise>
    memcpy( &amp;_dst_-><xsl:value-of select="member_name"/>, &amp;_src_-><xsl:value-of select="member_name"/> , <xsl:value-of select="arraysize"/>*sizeof(<xsl:value-of select="$member_type"/>) );
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
        <xsl:if test="arraytype='VARIABLE'">
    // copy variable array
          <xsl:choose>
            <xsl:when test="type='alias'">
    _dst_-><xsl:value-of select="member_name"/>_alloc(_src_-><xsl:value-of select="member_name"/>.size());
    for(int i=0;i&lt;_src_-><xsl:value-of select="member_name"/>.size();i++)
        copy( &amp;_dst_-><xsl:value-of select="member_name"/>()[i], &amp;_src_-><xsl:value-of select="member_name"/>[i] );
            </xsl:when>
            <xsl:otherwise>
    memcpy( _dst_-><xsl:value-of select="member_name"/>(), _src_-><xsl:value-of select="member_name"/>.data() , _src_-><xsl:value-of select="member_name"/>.size()*sizeof(<xsl:value-of select="$member_type"/>) );
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
        <xsl:if test="arraytype='NO'">
          <xsl:choose>
            <xsl:when test="type='alias'">
    copy( &amp;_dst_-><xsl:value-of select="member_name"/>, &amp;_src_-><xsl:value-of select="member_name"/> );
            </xsl:when>
            <xsl:otherwise>
    _dst_-><xsl:value-of select="member_name"/> = _src_-><xsl:value-of select="member_name"/>;
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
      </xsl:for-each>
    </xsl:if>
}
  </xsl:for-each>
</xsl:template>
</xsl:stylesheet>
