<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_proxy_class_id" />

<xsl:template match="/">
<xsl:variable name="component_name">
  <xsl:call-template name="replace">
    <xsl:with-param name="text" select="nerv_icde/component/name" />
    <xsl:with-param name="replace" select="' '" />
    <xsl:with-param name="by" select="'_'" />
  </xsl:call-template>
</xsl:variable>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row[id=$export_proxy_class_id]">
    <xsl:variable name="method_domain_id" select="method_domain_id" />
    <xsl:variable name="interface_name" select="interface_name" />
    <xsl:variable name="class_name" select="class_name" />

#include "<xsl:value-of select="$class_name"/>.h"
#include &lt;stdio.h&gt;
#include &lt;<xsl:value-of select="$component_name"/>/ros_data.hh&gt;
#include &lt;Nerv36/nerv_env.hh&gt;

    <xsl:for-each select="object_list/row">
      <xsl:variable name="object_name" select="object_name" />
class ROS_<xsl:value-of select="object_name"/> {
public:
    static <xsl:value-of select="$class_name"/>* proxy_;

      <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]/interface/row[name=$interface_name]/membermethod/row">
        <xsl:variable name="method_name" select="method_name" />
        <xsl:for-each select="../../../../method/row[name=$method_name]">
          <xsl:variable name="in_data_domain_path">
            <xsl:call-template name="get_data_domain_path">
              <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="out_data_domain_path">
            <xsl:call-template name="get_data_domain_path">
              <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="scope_in_path">
            <xsl:call-template name="replace">
              <xsl:with-param name="text" select="$in_data_domain_path" />
              <xsl:with-param name="replace" select="'/'" />
              <xsl:with-param name="by" select="'::'" />
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="scope_out_path">
            <xsl:call-template name="replace">
              <xsl:with-param name="text" select="$out_data_domain_path" />
              <xsl:with-param name="replace" select="'/'" />
              <xsl:with-param name="by" select="'::'" />
            </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="underscore_path">
            <xsl:call-template name="replace">
              <xsl:with-param name="text" select="$in_data_domain_path" />
              <xsl:with-param name="replace" select="'/'" />
              <xsl:with-param name="by" select="'__'" />
            </xsl:call-template>
          </xsl:variable>

          <xsl:variable name="ROS" select="starts-with($in_data_domain_path, '/ROS/')"/>
          <xsl:variable name="ROS_path">
            <xsl:call-template name="replace">
              <xsl:with-param name="text" select="$in_data_domain_path" />
              <xsl:with-param name="replace" select="'/ROS/'" />
              <xsl:with-param name="by" select="''" />
            </xsl:call-template>
          </xsl:variable>

          <xsl:choose>
            <xsl:when test="type='CALL'">
              <xsl:variable name="method_underscore_path">
                <xsl:call-template name="replace">
                  <xsl:with-param name="text" select="../../path" />
                  <xsl:with-param name="replace" select="'/'" />
                  <xsl:with-param name="by" select="'__'" />
                </xsl:call-template>
              </xsl:variable>
    static bool on_<xsl:value-of select="$method_name"/>
      (
        <xsl:value-of select="$component_name"/>::<xsl:value-of select="$method_name"/>::Request&amp; req,
        <xsl:value-of select="$component_name"/>::<xsl:value-of select="$method_name"/>::Response&amp; res
      )
    {
              <xsl:if test="in_data!='(null)'">
        auto cal_dat_ = Nerv36::Data<xsl:value-of select="$scope_in_path"/>::<xsl:value-of select="in_data"/>::malloc();
        copy( cal_dat_, &amp;req.dat );
              </xsl:if>
              <xsl:if test="out_data!='(null)'">
        auto ret_dat_ = Nerv36::Data<xsl:value-of select="$scope_out_path"/>::<xsl:value-of select="out_data"/>::malloc();
              </xsl:if>

              <xsl:if test="in_data='(null)' and out_data='(null)'">
        Nerv36::Error err = proxy_-&gt;<xsl:value-of select="$method_name"/>.call();
              </xsl:if>
              <xsl:if test="in_data='(null)' and out_data!='(null)'">
        Nerv36::Error err = proxy_-&gt;<xsl:value-of select="$method_name"/>.call( ret_dat_);
              </xsl:if>
              <xsl:if test="in_data!='(null)' and out_data='(null)'">
        Nerv36::Error err = proxy_-&gt;<xsl:value-of select="$method_name"/>.call( cal_dat_);
        cal_dat_-&gt;free();
              </xsl:if>
              <xsl:if test="in_data!='(null)' and out_data!='(null)'">
        Nerv36::Error err = proxy_-&gt;<xsl:value-of select="$method_name"/>.call( cal_dat_, ret_dat_);
        cal_dat_-&gt;free();
              </xsl:if>
        bool ret = (err == 0);
              <xsl:if test="out_data!='(null)'">
        if( ret ) copy( &amp;res.dat, ret_dat_ );
        ret_dat_-&gt;free();
              </xsl:if>
        return ret;
    }
            </xsl:when>
            <xsl:when test="type='SIGNAL'">
              <xsl:if test="in_data='(null)'">
    static void on_<xsl:value-of select="$method_name"/>(const <xsl:value-of select="$component_name"/>::null::ConstPtr&amp; msg)
    {
        proxy_-&gt;<xsl:value-of select="$method_name"/>.signal();
    }
              </xsl:if>
              <xsl:if test="in_data!='(null)'">
                <xsl:if test="not($ROS)"><xsl:text>
    </xsl:text>static void on_<xsl:value-of select="$method_name"/>(const <xsl:value-of select="$component_name"/>::<xsl:value-of select="in_data"/>::ConstPtr&amp; msg)<xsl:text/>
                </xsl:if>
                <xsl:if test="$ROS"><xsl:text>
    </xsl:text>static void on_<xsl:value-of select="$method_name"/>(const <xsl:value-of select="$ROS_path"/>::<xsl:value-of select="in_data"/>::ConstPtr&amp; msg)<xsl:text/>
                </xsl:if>
    {
        auto ptr = Nerv36::Data<xsl:value-of select="$scope_in_path"/>::<xsl:value-of select="in_data"/>::malloc();
        copy( ptr, msg.get() );
        proxy_-&gt;<xsl:value-of select="$method_name"/>.signal(ptr);
        ptr-&gt;free();
    }
              </xsl:if>
            </xsl:when>
            <xsl:when test="type='COMMAND'">
              <xsl:if test="in_data='(null)'">
    static void on_<xsl:value-of select="$method_name"/>(const <xsl:value-of select="$component_name"/>::null::ConstPtr&amp; msg)
    {
        proxy_-&gt;<xsl:value-of select="$method_name"/>.write();
    }
              </xsl:if>
              <xsl:if test="in_data!='(null)'">
                <xsl:if test="not($ROS)"><xsl:text>
    </xsl:text>static void on_<xsl:value-of select="$method_name"/>(const <xsl:value-of select="$component_name"/>::<xsl:value-of select="in_data"/>::ConstPtr&amp; msg)<xsl:text/>
                </xsl:if>
                <xsl:if test="$ROS"><xsl:text>
    </xsl:text>static void on_<xsl:value-of select="$method_name"/>(const <xsl:value-of select="$ROS_path"/>::<xsl:value-of select="in_data"/>::ConstPtr&amp; msg)<xsl:text/>
                </xsl:if>
    {
        auto ptr = Nerv36::Data<xsl:value-of select="$scope_in_path"/>::<xsl:value-of select="in_data"/>::malloc();
        copy( ptr, msg.get() );
        proxy_-&gt;<xsl:value-of select="$method_name"/>.write(ptr);
        ptr-&gt;free();
    }
              </xsl:if>
            </xsl:when>
          </xsl:choose>
        </xsl:for-each>
      </xsl:for-each>


    static void init(<xsl:value-of select="$class_name"/>* _proxy_, ros::NodeHandle* _ros_node_)
    {
        std::string topic_name;
        const char* obj_name_ = Nerv36::env("<xsl:value-of select="$object_name"/>", "<xsl:value-of select="$object_name"/>");
        const char* component_name = Nerv36::env("component_name","<xsl:value-of select="$component_name"/>");
        proxy_ = _proxy_;<xsl:text/>
      <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
        <xsl:for-each select="interface/row[name=$interface_name]">
          <xsl:for-each select="membermethod/row">
            <xsl:variable name="method_name" select="method_name" />
            <xsl:for-each select="../../../../method/row[name=$method_name]">
        topic_name = "/";
        topic_name += component_name;
        topic_name += "/";
        topic_name += obj_name_;
        topic_name += "/<xsl:value-of select="$method_name"/>";
              <xsl:choose>
                <xsl:when test="type='CALLBACK'">
                  <xsl:variable name="underscore_path">
                    <xsl:call-template name="replace">
                      <xsl:with-param name="text" select="../../path" />
                      <xsl:with-param name="replace" select="'/'" />
                      <xsl:with-param name="by" select="'__'" />
                    </xsl:call-template>
                  </xsl:variable><xsl:text>
        </xsl:text>proxy_-&gt;ros_<xsl:value-of select="$method_name"/>_ = new ros::ServiceClient(_ros_node_-&gt;serviceClient&lt;<xsl:value-of select="$component_name"/>::<xsl:value-of select="$method_name"/>&gt;( topic_name.c_str()));<xsl:text/>
                </xsl:when>
                <xsl:when test="type='SIGNALBACK' or type='EVENT' or type='INFORMATION' or type='BROADCAST' or type='DATAGRAM'">
                  <xsl:variable name="out_data_domain_path">
                    <xsl:call-template name="get_data_domain_path">
                      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:variable name="underscore_path">
                    <xsl:call-template name="replace">
                      <xsl:with-param name="text" select="$out_data_domain_path" />
                      <xsl:with-param name="replace" select="'/'" />
                      <xsl:with-param name="by" select="'__'" />
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:variable name="ROS" select="starts-with($out_data_domain_path, '/ROS/')"/>
                  <xsl:variable name="ROS_path">
                    <xsl:call-template name="replace">
                      <xsl:with-param name="text" select="$out_data_domain_path" />
                      <xsl:with-param name="replace" select="'/ROS/'" />
                      <xsl:with-param name="by" select="''" />
                    </xsl:call-template>
                  </xsl:variable>
                  <xsl:if test="out_data='(null)'"><xsl:text>
        </xsl:text>proxy_-&gt;ros_<xsl:value-of select="$method_name"/>_ = new ros::Publisher(_ros_node_-&gt;advertise&lt;<xsl:value-of select="$component_name"/>::null&gt;( topic_name.c_str(), 1));<xsl:text/>
                  </xsl:if>
                  <xsl:if test="out_data!='(null)'">
                    <xsl:if test="not($ROS)"><xsl:text>
        </xsl:text>proxy_-&gt;ros_<xsl:value-of select="$method_name"/>_ = new ros::Publisher(_ros_node_-&gt;advertise&lt;<xsl:value-of select="$component_name"/>::<xsl:value-of select="out_data"/>&gt;( topic_name.c_str(), 1));<xsl:text/>
                    </xsl:if>
                    <xsl:if test="$ROS"><xsl:text>
        </xsl:text>proxy_-&gt;ros_<xsl:value-of select="$method_name"/>_ = new ros::Publisher(_ros_node_-&gt;advertise&lt;<xsl:value-of select="$ROS_path"/>::<xsl:value-of select="out_data"/>&gt;( topic_name.c_str(), 1));<xsl:text/>
                    </xsl:if>
                  </xsl:if>
                </xsl:when>
                <xsl:when test="type='CALL'"><xsl:text>
        </xsl:text>proxy_-&gt;ros_<xsl:value-of select="$method_name"/>_ = new ros::ServiceServer(_ros_node_-&gt;advertiseService( topic_name.c_str(), ROS_<xsl:value-of select="$object_name"/>::on_<xsl:value-of select="$method_name"/> ));<xsl:text/>
                </xsl:when>
                <xsl:when test="type='SIGNAL' or type='COMMAND'"><xsl:text>
        </xsl:text>proxy_-&gt;ros_<xsl:value-of select="$method_name"/>_ = new ros::Subscriber(_ros_node_-&gt;subscribe( topic_name.c_str(), 1, ROS_<xsl:value-of select="$object_name"/>::on_<xsl:value-of select="$method_name"/> ));<xsl:text/>
                </xsl:when>
              </xsl:choose>
            </xsl:for-each>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    }
};
<xsl:value-of select="$class_name"/>* ROS_<xsl:value-of select="$object_name"/>::proxy_ = 0;<xsl:text/>
    </xsl:for-each>
<xsl:text/>
<xsl:value-of select="$class_name"/>::<xsl:value-of select="$class_name"/>( std::string _obj_name, ros::NodeHandle* _ros_node_)
    : Proxy_<xsl:value-of select="$interface_name"/>(this)
    , ros_node_(_ros_node_)
    , obj_name(_obj_name)
{
    <xsl:for-each select="object_list/row">
    if( _obj_name == "<xsl:value-of select="object_name"/>" ) {
        ROS_<xsl:value-of select="object_name"/>::init(this, ros_node_);
    }
    </xsl:for-each>
}

<xsl:value-of select="$class_name"/>::~<xsl:value-of select="$class_name"/>()
{
}

Nerv36::Error <xsl:value-of select="$class_name"/>::start()
{
    const char* obj_name_ = Nerv36::env(obj_name.c_str(), obj_name.c_str());
    const char* component_name_ = Nerv36::env("component_name","<xsl:value-of select="$component_name"/>");
    std::string stub_id = component_name_;
    stub_id += "/";
    stub_id += obj_name_;

    Nerv36::Error err = Proxy::start(stub_id.c_str());
    printf("%s.proxy.start() = %s\n", stub_id.c_str(), Nerv36::get_err_str(err));
    return err;
}

void <xsl:value-of select="$class_name"/>::on_connect()
{
    printf("<xsl:value-of select="$class_name"/>::on_connect\n");
    Nerv36::Error err;
    Nerv36::Msg_num msg_num;

    double connect_timeout = 1.0;
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:if test="type='COMMAND' or type='EVENT' or type='INFORMATION'">
    msg_num = 0;
    printf("<xsl:value-of select="$class_name"/>::<xsl:value-of select="name"/>_delegate_link()\n");<xsl:text/>
    err = <xsl:value-of select="name"/>.delegate_link(&amp;msg_num, <xsl:if test="type='INFORMATION'">false, </xsl:if> connect_timeout );
    printf("<xsl:value-of select="$class_name"/>::<xsl:value-of select="name"/>_delegate_link(%d) = %s\n" , msg_num, Nerv36::get_err_str(err) );<xsl:text/>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>

    // ROS에는 connect 개념이 없으므로 사용되지 아니함.
}

void <xsl:value-of select="$class_name"/>::on_disconnect()
{
    printf("<xsl:value-of select="$class_name"/>::on_disconnect\n");
    stop();
    // ROS에는 connect 개념이 없으므로 사용되지 아니함.
}

    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="underscore_path">
              <xsl:call-template name="replace">
                <xsl:with-param name="text" select="$out_data_domain_namespace" />
                <xsl:with-param name="replace" select="'::'" />
                <xsl:with-param name="by" select="'__'" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="ROS" select="starts-with($out_data_domain_namespace, '::ROS::')"/>
            <xsl:variable name="ROS_path">
              <xsl:call-template name="replace">
                <xsl:with-param name="text" select="$out_data_domain_namespace" />
                <xsl:with-param name="replace" select="'::ROS::'" />
                <xsl:with-param name="by" select="''" />
              </xsl:call-template>
            </xsl:variable>

            // hello <xsl:value-of select="$out_data_domain_namespace"/>

            <xsl:choose>
              <xsl:when test="type='CALLBACK'">
                <xsl:variable name="underscore_method_path">
                  <xsl:call-template name="replace">
                    <xsl:with-param name="text" select="../../path" />
                    <xsl:with-param name="replace" select="'/'" />
                    <xsl:with-param name="by" select="'__'" />
                  </xsl:call-template>
                </xsl:variable>
Nerv36::Error <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>( <xsl:if test="out_data!='(null)'">const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _clb_dat_, </xsl:if><xsl:if test="in_data!='(null)'">Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _ret_dat_, </xsl:if> double _elapsed, double _limit)
{
    printf("<xsl:value-of select="$class_name"/>::callback::on_<xsl:value-of select="$method_name"/>\n");
    auto srv_ = new <xsl:value-of select="$component_name"/>::<xsl:value-of select="$method_name"/>;
                <xsl:if test="out_data!='(null)'">
    copy( &amp;srv_-&gt;request.dat, _clb_dat_ );
                </xsl:if>
    bool ret = ros_<xsl:value-of select="$method_name"/>_-&gt;call(*srv_);
                <xsl:if test="in_data!='(null)'">
    if(ret) copy( &amp;srv_-&gt;response.dat, _ret_dat_ );
                </xsl:if>
    delete srv_;
    return ret ? 0 : 0xFFFF;
}
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_cancel( Nerv36::Error _err)
{
    printf("<xsl:value-of select="$class_name"/>::callback::on_<xsl:value-of select="$method_name"/>_cancel\n");
    // ROS에는 Cancel 개념이 없으므로 사용되지 아니함.
}
              </xsl:when>
              <xsl:when test="type='SIGNALBACK'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>( <xsl:if test="out_data!='(null)'">const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _sgb_dat_,</xsl:if> double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::signalback::on_<xsl:value-of select="$method_name"/>\n");

    <xsl:if test="out_data='(null)'">
    auto dat_ = new <xsl:value-of select="$component_name"/>::null;
    </xsl:if>
    <xsl:if test="out_data!='(null)'">
      <xsl:if test="not($ROS)"><xsl:text>
    </xsl:text>auto dat_ = new <xsl:value-of select="$component_name"/>::<xsl:value-of select="out_data"/>;<xsl:text/>
      </xsl:if>
      <xsl:if test="$ROS"><xsl:text>
    </xsl:text>auto dat_ = new <xsl:value-of select="$ROS_path"/>::<xsl:value-of select="out_data"/>;<xsl:text/>
      </xsl:if>
    copy( dat_, _sgb_dat_ );
    </xsl:if>
    ros_<xsl:value-of select="$method_name"/>_-&gt;publish(*dat_);
    delete dat_;
}
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_cancel( Nerv36::Error _err)
{
    printf("<xsl:value-of select="$class_name"/>::signalback::on_<xsl:value-of select="$method_name"/>_cancel\n");
    // ROS에는 Cancel 개념이 없으므로 사용되지 아니함.
}
              </xsl:when>
              <xsl:when test="type='CALL'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_return(Nerv36::Msg_num _msg_num, Nerv36::Error _err,<xsl:if test="out_data!='(null)'"> const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _ret_dat_,</xsl:if> double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::call::on_<xsl:value-of select="$method_name"/>_return\n");
    // ROS변환에는 aync-await 방식을 사용하게되므로 delegate 방식은 사용되지 아니함.
}
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_return(Nerv36::Msg_num _msg_num, Nerv36::Error _err, double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::signal::on_<xsl:value-of select="$method_name"/>_return\n");
    // ROS변환에는 aync-await 방식을 사용하게되므로 delegate 방식은 사용되지 아니함.
}
              </xsl:when>
              <xsl:when test="type='EVENT'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_link(Nerv36::Msg_num _msg_num, Nerv36::Error _err, double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::event::on_<xsl:value-of select="$method_name"/>_link\n");
    // ROS에는 Link 개념이 없으므로 사용되지 아니함.
}

void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink(Nerv36::Error _err)
{
    printf("<xsl:value-of select="$class_name"/>::event::on_<xsl:value-of select="$method_name"/>_unlink\n");
    // ROS에는 Link 개념이 없으므로 사용되지 아니함.
}
                <xsl:choose >
                  <xsl:when test="out_data='(null)'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>( double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::event::on_<xsl:value-of select="$method_name"/>\n");
    auto dat_ = new <xsl:value-of select="$component_name"/>::null;
    ros_<xsl:value-of select="$method_name"/>_-&gt;publish(*dat_);
    delete dat_;
}
                  </xsl:when>
                  <xsl:otherwise>
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>(const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _evt_dat_, double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::event::on_<xsl:value-of select="$method_name"/>\n");
      <xsl:if test="not($ROS)"><xsl:text>
    </xsl:text>auto dat_ = new <xsl:value-of select="$component_name"/>::<xsl:value-of select="out_data"/>;<xsl:text/>
      </xsl:if>
      <xsl:if test="$ROS"><xsl:text>
    </xsl:text>auto dat_ = new <xsl:value-of select="$ROS_path"/>::<xsl:value-of select="out_data"/>;<xsl:text/>
      </xsl:if>
    copy( dat_, _evt_dat_ );
    ros_<xsl:value-of select="$method_name"/>_-&gt;publish(*dat_);
    delete dat_;
}
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>


              <xsl:when test="type='COMMAND'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_link(Nerv36::Msg_num _msg_num, Nerv36::Error _err, double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::command::on_<xsl:value-of select="$method_name"/>_link\n");
    // ROS에는 Link 개념이 없으므로 사용되지 아니함.
}
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink(Nerv36::Error _err)
{
    printf("<xsl:value-of select="$class_name"/>::command::on_<xsl:value-of select="$method_name"/>_unlink\n");
    // ROS에는 Link 개념이 없으므로 사용되지 아니함.
}
              </xsl:when>


              <xsl:when test="type='INFORMATION'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_link(Nerv36::Msg_num _msg_num, Nerv36::Error _err, double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::information::on_<xsl:value-of select="$method_name"/>_link\n");
    // ROS에는 Link 개념이 없으므로 사용되지 아니함.
}
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink(Nerv36::Error _err)
{
    printf("<xsl:value-of select="$class_name"/>::information::on_<xsl:value-of select="$method_name"/>_unlink\n");
    // ROS에는 Link 개념이 없으므로 사용되지 아니함.
}
                <xsl:choose >
                  <xsl:when test="out_data='(null)'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_sync( Nerv36::Sync_model _sync_model, double _elapsed, double _limit )
{
    printf("<xsl:value-of select="$class_name"/>::information::on_<xsl:value-of select="$method_name"/>_sync\n");
    auto dat_ = new <xsl:value-of select="$component_name"/>::null;
    ros_<xsl:value-of select="$method_name"/>_-&gt;publish(*dat_);
    delete dat_;
}
                  </xsl:when>
                  <xsl:otherwise>
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_sync( Nerv36::Sync_model _sync_model, double _elapsed, double _limit, const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _inf_dat_ )
{
    printf("<xsl:value-of select="$class_name"/>::information::on_<xsl:value-of select="$method_name"/>_sync\n");
      <xsl:if test="not($ROS)"><xsl:text>
    </xsl:text>auto dat_ = new <xsl:value-of select="$component_name"/>::<xsl:value-of select="out_data"/>;<xsl:text/>
      </xsl:if>
      <xsl:if test="$ROS"><xsl:text>
    </xsl:text>auto dat_ = new <xsl:value-of select="$ROS_path"/>::<xsl:value-of select="out_data"/>;<xsl:text/>
      </xsl:if>
    // hello <xsl:value-of select="$ROS"/>
    copy( dat_, _inf_dat_ );
    ros_<xsl:value-of select="$method_name"/>_-&gt;publish(*dat_);
    delete dat_;
}
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:when test="type='BROADCAST'">
                <xsl:choose >
                  <xsl:when test="out_data='(null)'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>( Nerv36::Address _stub_addr, double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::datagram::on_<xsl:value-of select="$method_name"/>\n");
    auto dat_ = new <xsl:value-of select="$component_name"/>::null;
    ros_<xsl:value-of select="$method_name"/>_-&gt;publish(*dat_);
    delete dat_;
}
                  </xsl:when>
                  <xsl:otherwise>
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>( Nerv36::Address _stub_addr, const Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _bst_dat_, double _elapsed)
{
    printf("<xsl:value-of select="$class_name"/>::datagram::on_<xsl:value-of select="$method_name"/>\n");
      <xsl:if test="not($ROS)"><xsl:text>
    </xsl:text>auto dat_ = new <xsl:value-of select="$component_name"/>::<xsl:value-of select="out_data"/>;<xsl:text/>
      </xsl:if>
      <xsl:if test="$ROS"><xsl:text>
    </xsl:text>auto dat_ = new <xsl:value-of select="$ROS_path"/>::<xsl:value-of select="out_data"/>;<xsl:text/>
      </xsl:if>
    copy( dat_, _bst_dat_ );
    ros_<xsl:value-of select="$method_name"/>_-&gt;publish(*dat_);
    delete dat_;
}
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>
