// 이 파일은 Nerv의

#include "nerv_define.h"

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#if defined __cplusplus
extern "C" {
#endif

typedef int32_t  Error;
typedef uint32_t Nerv_ip;
typedef uint16_t Nerv_port;
typedef int      Nerv_inst;

typedef struct {
    Nerv_ip   ip;
    Nerv_port port;
    const char* id;
} Nerv_addr;

typedef uint16_t Nerv_code;

typedef uint32_t Nerv_seq;

typedef uint32_t Nerv_pksize;

typedef uint8_t       Nerv_sync_model;
const Nerv_sync_model nerv_on_post = 0x00U;
const Nerv_sync_model nerv_on_timer = 0x01U;

// Stub delegate
typedef void on_nerv_stub_connect_ft(Nerv_inst* _object_,
                                     Nerv_addr  _proxy_addr);
typedef void on_nerv_stub_disconnect_ft(Nerv_inst* _object_,
                                        Nerv_addr  _proxy_addr);

// Stub_call delegate
typedef Error on_nerv_stub_call_ft(Nerv_inst*   _call_,
                                   Nerv_addr    _proxy_addr,
                                   const void*  _in_buf_,
                                   Nerv_pksize  _in_buf_size,
                                   void*        _out_buf_,
                                   Nerv_pksize* _out_buf_size_,
                                   double       _elapsed,
                                   double       _limit);

typedef void on_nerv_stub_call_cancel_ft(Nerv_inst* _call_,
                                         Nerv_addr  _proxy_addr,
                                         Error      _error);

// Stub_callback delegate
typedef void on_nerv_stub_callback_return_ft(Nerv_inst*  _callback_,
                                             Nerv_seq    _seq,
                                             Nerv_addr   _proxy_addr,
                                             Error       _err,
                                             const void* _buf_,
                                             Nerv_pksize _size,
                                             double      _time);

// Stub_command delegate
typedef void on_nerv_stub_command_sync_ft(Nerv_inst*      _command_,
                                          Nerv_addr       _proxy_addr,
                                          Nerv_sync_model _sync_model,
                                          double          _elapsed,
                                          double          _limit,
                                          const void*     _buffer_,
                                          Nerv_pksize     _size);

typedef Error on_nerv_stub_command_link_ft(Nerv_inst* _command_,
                                           Nerv_addr  _proxy_addr,
                                           double     _elapsed,
                                           double     _limit);

typedef void on_nerv_stub_command_unlink_ft(Nerv_inst* _object_,
                                            Nerv_addr  _proxy_addr,
                                            Error      _err);

// Stub_event delegate
typedef Error on_nerv_stub_event_link_ft(Nerv_inst* _event_,
                                         Nerv_addr  _proxy_addr,
                                         double     _elapsed,
                                         double     _limit);

typedef void on_nerv_stub_event_unlink_ft(Nerv_inst* _event_,
                                          Nerv_addr  _proxy_addr,
                                          Error      _err);

// Stub_information delegate
typedef Error on_nerv_stub_information_link_ft(Nerv_inst* _information_,
                                               Nerv_addr  _proxy_addr,
                                               bool       _reliability,
                                               double     _elapsed,
                                               double     _limit);

typedef void on_nerv_stub_information_unlink_ft(Nerv_inst* _information_,
                                                Nerv_addr  _proxy_addr,
                                                Error      _err);

// Stub_signal delegate
typedef void on_nerv_stub_signal_ft(Nerv_inst*  _signal_,
                                    Nerv_addr   _proxy_addr,
                                    const void* _in_buf_,
                                    Nerv_pksize _in_buf_size,
                                    double      _elapsed);

typedef void on_nerv_stub_signal_cancel_ft(Nerv_inst* _signal_,
                                           Nerv_addr  _proxy_addr,
                                           Error      _error);

// Stub_signalback delegate
typedef void on_nerv_stub_signalback_return_ft(Nerv_inst* _signal_,
                                               Nerv_seq   _seq,
                                               Nerv_addr  _proxy_addr,
                                               Error      _err,
                                               double     _time);

// Proxy delegate
typedef void on_nerv_proxy_connect_ft(Nerv_inst* _object_);
typedef void on_nerv_proxy_disconnect_ft(Nerv_inst* _object_);

// Prox_call delegate
typedef void on_nerv_proxy_call_return_ft(Nerv_inst*  _call_,
                                          Nerv_seq    _seq,
                                          Error       _err,
                                          const void* _buf_,
                                          Nerv_pksize _size,
                                          double      _time);

// Proxy_callback delegate
typedef Error on_nerv_proxy_callback_ft(Nerv_inst*   _callback_,
                                        const void*  _in_buf_,
                                        Nerv_pksize  _in_buf_size,
                                        void*        _out_buf_,
                                        Nerv_pksize* _out_buf_size_,
                                        double       _elapsed,
                                        double       _limit);

typedef void on_nerv_proxy_callback_cancel_ft(Nerv_inst* _callback_,
                                              Error      _error);

// Prox_signal delegate
typedef void on_nerv_proxy_signal_return_ft(Nerv_inst* _signal_,
                                            Nerv_seq   _seq,
                                            Error      _err,
                                            double     _time);

// Proxy_signalback delegate
typedef void on_nerv_proxy_signalback_ft(Nerv_inst*  _signalback_,
                                         const void* _in_buf_,
                                         Nerv_pksize _in_buf_size,
                                         double      _elapsed);

typedef void on_nerv_proxy_signalback_cancel_ft(Nerv_inst* _signalback_,
                                                Error      _error);

// Proxy_command delegate
typedef void on_nerv_proxy_command_unlink_ft(Nerv_inst* _command_, Error _err);

typedef void on_nerv_proxy_command_link_ft(Nerv_inst* _command_,
                                           Nerv_seq   _seq,
                                           Error      _err,
                                           double     _time);

// Proxy_datagram delegate
typedef void on_nerv_proxy_datagram_cast_ft(Nerv_inst*  _datagram_,
                                            Nerv_addr   _stub_addr,
                                            const void* _buf_,
                                            Nerv_pksize _size,
                                            double      _elapsed);

// Proxy_event delegate
typedef void on_nerv_proxy_event_unlink_ft(Nerv_inst* _event_, Error _err);

typedef void on_nerv_proxy_event_ft(Nerv_inst*  _event_,
                                    const void* _event_data_,
                                    Nerv_pksize _event_size,
                                    double      _elapsed);

typedef void on_nerv_proxy_event_link_ft(Nerv_inst* _event_,
                                         Nerv_seq   _seq,
                                         Error      _err,
                                         double     _time);

// Proxy_information delegate
typedef void on_nerv_proxy_information_unlink_ft(Nerv_inst* _information_,
                                                 Error      _err);

typedef void on_nerv_proxy_information_sync_ft(Nerv_inst*      _information_,
                                               Nerv_sync_model _sync_model,
                                               double          _elapsed,
                                               double          _limit,
                                               const void*     _buffer_,
                                               Nerv_pksize     _size);

typedef void on_nerv_proxy_information_link_ft(Nerv_inst* _information_,
                                               Nerv_seq   _seq,
                                               Error      _err,
                                               double     _time);

#if defined __cplusplus
} // extern "C" {
#endif
